//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct NSTextCheckingController;

    unsafe impl ClassType for NSTextCheckingController {
        type Super = NSObject;
        type Mutability = InteriorMutable;
    }
);

unsafe impl NSObjectProtocol for NSTextCheckingController {}

extern_methods!(
    unsafe impl NSTextCheckingController {
        #[cfg(all(feature = "NSTextCheckingClient", feature = "NSTextInputClient"))]
        #[method_id(@__retain_semantics Init initWithClient:)]
        pub unsafe fn initWithClient(
            this: Allocated<Self>,
            client: &ProtocolObject<dyn NSTextCheckingClient>,
        ) -> Retained<Self>;

        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[cfg(all(feature = "NSTextCheckingClient", feature = "NSTextInputClient"))]
        #[method_id(@__retain_semantics Other client)]
        pub unsafe fn client(&self) -> Retained<ProtocolObject<dyn NSTextCheckingClient>>;

        #[method(invalidate)]
        pub unsafe fn invalidate(&self);

        #[method(didChangeTextInRange:)]
        pub unsafe fn didChangeTextInRange(&self, range: NSRange);

        #[method(insertedTextInRange:)]
        pub unsafe fn insertedTextInRange(&self, range: NSRange);

        #[method(didChangeSelectedRange)]
        pub unsafe fn didChangeSelectedRange(&self);

        #[method(considerTextCheckingForRange:)]
        pub unsafe fn considerTextCheckingForRange(&self, range: NSRange);

        #[cfg(feature = "NSSpellChecker")]
        #[method(checkTextInRange:types:options:)]
        pub unsafe fn checkTextInRange_types_options(
            &self,
            range: NSRange,
            checking_types: NSTextCheckingTypes,
            options: &NSDictionary<NSTextCheckingOptionKey, AnyObject>,
        );

        #[method(checkTextInSelection:)]
        pub unsafe fn checkTextInSelection(&self, sender: Option<&AnyObject>);

        #[method(checkTextInDocument:)]
        pub unsafe fn checkTextInDocument(&self, sender: Option<&AnyObject>);

        #[method(orderFrontSubstitutionsPanel:)]
        pub unsafe fn orderFrontSubstitutionsPanel(&self, sender: Option<&AnyObject>);

        #[method(checkSpelling:)]
        pub unsafe fn checkSpelling(&self, sender: Option<&AnyObject>);

        #[method(showGuessPanel:)]
        pub unsafe fn showGuessPanel(&self, sender: Option<&AnyObject>);

        #[method(changeSpelling:)]
        pub unsafe fn changeSpelling(&self, sender: Option<&AnyObject>);

        #[method(ignoreSpelling:)]
        pub unsafe fn ignoreSpelling(&self, sender: Option<&AnyObject>);

        #[method(updateCandidates)]
        pub unsafe fn updateCandidates(&self);

        #[method_id(@__retain_semantics Other validAnnotations)]
        pub unsafe fn validAnnotations(&self) -> Retained<NSArray<NSAttributedStringKey>>;

        #[cfg(feature = "NSMenu")]
        #[method_id(@__retain_semantics Other menuAtIndex:clickedOnSelection:effectiveRange:)]
        pub unsafe fn menuAtIndex_clickedOnSelection_effectiveRange(
            &self,
            location: NSUInteger,
            clicked_on_selection: bool,
            effective_range: NSRangePointer,
            mtm: MainThreadMarker,
        ) -> Option<Retained<NSMenu>>;

        #[method(spellCheckerDocumentTag)]
        pub unsafe fn spellCheckerDocumentTag(&self) -> NSInteger;

        #[method(setSpellCheckerDocumentTag:)]
        pub unsafe fn setSpellCheckerDocumentTag(&self, spell_checker_document_tag: NSInteger);
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl NSTextCheckingController {
        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new() -> Retained<Self>;
    }
);
