//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_category!(
    /// Category "NSExtensions" on [`NSFileWrapper`].
    #[doc(alias = "NSExtensions")]
    pub unsafe trait NSFileWrapperNSExtensions {
        #[cfg(feature = "NSImage")]
        #[method_id(@__retain_semantics Other icon)]
        unsafe fn icon(&self) -> Option<Retained<NSImage>>;

        #[cfg(feature = "NSImage")]
        #[method(setIcon:)]
        unsafe fn setIcon(&self, icon: Option<&NSImage>);
    }

    unsafe impl NSFileWrapperNSExtensions for NSFileWrapper {}
);
