//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ffi::*;
use core::ptr::NonNull;
use objc2::__framework_prelude::*;
#[cfg(feature = "objc2-core-foundation")]
use objc2_core_foundation::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    /// [Apple's documentation](https://developer.apple.com/documentation/coreimage/cifiltershape?language=objc)
    #[unsafe(super(NSObject))]
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct CIFilterShape;
);

extern_conformance!(
    unsafe impl NSCopying for CIFilterShape {}
);

unsafe impl CopyingHelper for CIFilterShape {
    type Result = Self;
}

extern_conformance!(
    unsafe impl NSObjectProtocol for CIFilterShape {}
);

impl CIFilterShape {
    extern_methods!(
        #[cfg(feature = "objc2-core-foundation")]
        #[unsafe(method(shapeWithRect:))]
        #[unsafe(method_family = none)]
        pub unsafe fn shapeWithRect(r: CGRect) -> Retained<Self>;

        #[cfg(feature = "objc2-core-foundation")]
        #[unsafe(method(initWithRect:))]
        #[unsafe(method_family = init)]
        pub unsafe fn initWithRect(this: Allocated<Self>, r: CGRect) -> Retained<Self>;

        #[cfg(feature = "objc2-core-foundation")]
        #[unsafe(method(transformBy:interior:))]
        #[unsafe(method_family = none)]
        pub unsafe fn transformBy_interior(
            &self,
            m: CGAffineTransform,
            flag: bool,
        ) -> Retained<CIFilterShape>;

        #[unsafe(method(insetByX:Y:))]
        #[unsafe(method_family = none)]
        pub unsafe fn insetByX_Y(&self, dx: c_int, dy: c_int) -> Retained<CIFilterShape>;

        #[unsafe(method(unionWith:))]
        #[unsafe(method_family = none)]
        pub unsafe fn unionWith(&self, s2: &CIFilterShape) -> Retained<CIFilterShape>;

        #[cfg(feature = "objc2-core-foundation")]
        #[unsafe(method(unionWithRect:))]
        #[unsafe(method_family = none)]
        pub unsafe fn unionWithRect(&self, r: CGRect) -> Retained<CIFilterShape>;

        #[unsafe(method(intersectWith:))]
        #[unsafe(method_family = none)]
        pub unsafe fn intersectWith(&self, s2: &CIFilterShape) -> Retained<CIFilterShape>;

        #[cfg(feature = "objc2-core-foundation")]
        #[unsafe(method(intersectWithRect:))]
        #[unsafe(method_family = none)]
        pub unsafe fn intersectWithRect(&self, r: CGRect) -> Retained<CIFilterShape>;

        #[cfg(feature = "objc2-core-foundation")]
        #[unsafe(method(extent))]
        #[unsafe(method_family = none)]
        pub unsafe fn extent(&self) -> CGRect;
    );
}

/// Methods declared on superclass `NSObject`.
impl CIFilterShape {
    extern_methods!(
        #[unsafe(method(init))]
        #[unsafe(method_family = init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[unsafe(method(new))]
        #[unsafe(method_family = new)]
        pub unsafe fn new() -> Retained<Self>;
    );
}
