//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ffi::*;
use core::ptr::NonNull;
use objc2::__framework_prelude::*;

use crate::*;

extern_protocol!(
    /// [Apple's documentation](https://developer.apple.com/documentation/foundation/nsextensionrequesthandling?language=objc)
    pub unsafe trait NSExtensionRequestHandling: NSObjectProtocol {
        #[cfg(feature = "NSExtensionContext")]
        #[unsafe(method(beginRequestWithExtensionContext:))]
        #[unsafe(method_family = none)]
        fn beginRequestWithExtensionContext(&self, context: &NSExtensionContext);
    }
);
