//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ffi::*;
use core::ptr::NonNull;
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityerrorcodeexceptioninfo?language=objc)
    pub static NSAccessibilityErrorCodeExceptionInfo: &'static NSString;
}

/// * Accessibility Constants **
///
/// See also [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityattributename?language=objc)
// NS_TYPED_ENUM
pub type NSAccessibilityAttributeName = NSString;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityroleattribute?language=objc)
    pub static NSAccessibilityRoleAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityroledescriptionattribute?language=objc)
    pub static NSAccessibilityRoleDescriptionAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysubroleattribute?language=objc)
    pub static NSAccessibilitySubroleAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityhelpattribute?language=objc)
    pub static NSAccessibilityHelpAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityvalueattribute?language=objc)
    pub static NSAccessibilityValueAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityminvalueattribute?language=objc)
    pub static NSAccessibilityMinValueAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymaxvalueattribute?language=objc)
    pub static NSAccessibilityMaxValueAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityenabledattribute?language=objc)
    pub static NSAccessibilityEnabledAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfocusedattribute?language=objc)
    pub static NSAccessibilityFocusedAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityparentattribute?language=objc)
    pub static NSAccessibilityParentAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitychildrenattribute?language=objc)
    pub static NSAccessibilityChildrenAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitywindowattribute?language=objc)
    pub static NSAccessibilityWindowAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytopleveluielementattribute?language=objc)
    pub static NSAccessibilityTopLevelUIElementAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityselectedchildrenattribute?language=objc)
    pub static NSAccessibilitySelectedChildrenAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityvisiblechildrenattribute?language=objc)
    pub static NSAccessibilityVisibleChildrenAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitypositionattribute?language=objc)
    pub static NSAccessibilityPositionAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysizeattribute?language=objc)
    pub static NSAccessibilitySizeAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycontentsattribute?language=objc)
    pub static NSAccessibilityContentsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytitleattribute?language=objc)
    pub static NSAccessibilityTitleAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydescriptionattribute?language=objc)
    pub static NSAccessibilityDescriptionAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityshownmenuattribute?language=objc)
    pub static NSAccessibilityShownMenuAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityvaluedescriptionattribute?language=objc)
    pub static NSAccessibilityValueDescriptionAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysharedfocuselementsattribute?language=objc)
    pub static NSAccessibilitySharedFocusElementsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitypreviouscontentsattribute?language=objc)
    pub static NSAccessibilityPreviousContentsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitynextcontentsattribute?language=objc)
    pub static NSAccessibilityNextContentsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityheaderattribute?language=objc)
    pub static NSAccessibilityHeaderAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityeditedattribute?language=objc)
    pub static NSAccessibilityEditedAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytabsattribute?language=objc)
    pub static NSAccessibilityTabsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityhorizontalscrollbarattribute?language=objc)
    pub static NSAccessibilityHorizontalScrollBarAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityverticalscrollbarattribute?language=objc)
    pub static NSAccessibilityVerticalScrollBarAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityoverflowbuttonattribute?language=objc)
    pub static NSAccessibilityOverflowButtonAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityincrementbuttonattribute?language=objc)
    pub static NSAccessibilityIncrementButtonAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydecrementbuttonattribute?language=objc)
    pub static NSAccessibilityDecrementButtonAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfilenameattribute?language=objc)
    pub static NSAccessibilityFilenameAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityexpandedattribute?language=objc)
    pub static NSAccessibilityExpandedAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityselectedattribute?language=objc)
    pub static NSAccessibilitySelectedAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysplittersattribute?language=objc)
    pub static NSAccessibilitySplittersAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydocumentattribute?language=objc)
    pub static NSAccessibilityDocumentAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityactivationpointattribute?language=objc)
    pub static NSAccessibilityActivationPointAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityurlattribute?language=objc)
    pub static NSAccessibilityURLAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityindexattribute?language=objc)
    pub static NSAccessibilityIndexAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrowcountattribute?language=objc)
    pub static NSAccessibilityRowCountAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycolumncountattribute?language=objc)
    pub static NSAccessibilityColumnCountAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityorderedbyrowattribute?language=objc)
    pub static NSAccessibilityOrderedByRowAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitywarningvalueattribute?language=objc)
    pub static NSAccessibilityWarningValueAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycriticalvalueattribute?language=objc)
    pub static NSAccessibilityCriticalValueAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityplaceholdervalueattribute?language=objc)
    pub static NSAccessibilityPlaceholderValueAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycontainsprotectedcontentattribute?language=objc)
    pub static NSAccessibilityContainsProtectedContentAttribute:
        &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityalternateuivisibleattribute?language=objc)
    pub static NSAccessibilityAlternateUIVisibleAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrequiredattribute?language=objc)
    pub static NSAccessibilityRequiredAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityautointeractableattribute?language=objc)
    pub static NSAccessibilityAutoInteractableAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydatetimecomponentsattribute?language=objc)
    pub static NSAccessibilityDateTimeComponentsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityembeddedimagedescriptionattribute?language=objc)
    pub static NSAccessibilityEmbeddedImageDescriptionAttribute:
        &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitypathattribute?language=objc)
    pub static NSAccessibilityPathAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytextinputmarkedrangeattribute?language=objc)
    pub static NSAccessibilityTextInputMarkedRangeAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityblockquotelevelattribute?language=objc)
    pub static NSAccessibilityBlockQuoteLevelAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityheadinglevelattribute?language=objc)
    pub static NSAccessibilityHeadingLevelAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylanguageattribute?language=objc)
    pub static NSAccessibilityLanguageAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityvisitedattribute?language=objc)
    pub static NSAccessibilityVisitedAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytitleuielementattribute?language=objc)
    pub static NSAccessibilityTitleUIElementAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityservesastitleforuielementsattribute?language=objc)
    pub static NSAccessibilityServesAsTitleForUIElementsAttribute:
        &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylinkeduielementsattribute?language=objc)
    pub static NSAccessibilityLinkedUIElementsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityselectedtextattribute?language=objc)
    pub static NSAccessibilitySelectedTextAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityselectedtextrangeattribute?language=objc)
    pub static NSAccessibilitySelectedTextRangeAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitynumberofcharactersattribute?language=objc)
    pub static NSAccessibilityNumberOfCharactersAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityvisiblecharacterrangeattribute?language=objc)
    pub static NSAccessibilityVisibleCharacterRangeAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysharedtextuielementsattribute?language=objc)
    pub static NSAccessibilitySharedTextUIElementsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysharedcharacterrangeattribute?language=objc)
    pub static NSAccessibilitySharedCharacterRangeAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityinsertionpointlinenumberattribute?language=objc)
    pub static NSAccessibilityInsertionPointLineNumberAttribute:
        &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityselectedtextrangesattribute?language=objc)
    pub static NSAccessibilitySelectedTextRangesAttribute: &'static NSAccessibilityAttributeName;
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityparameterizedattributename?language=objc)
// NS_TYPED_ENUM
pub type NSAccessibilityParameterizedAttributeName = NSString;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylineforindexparameterizedattribute?language=objc)
    pub static NSAccessibilityLineForIndexParameterizedAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrangeforlineparameterizedattribute?language=objc)
    pub static NSAccessibilityRangeForLineParameterizedAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitystringforrangeparameterizedattribute?language=objc)
    pub static NSAccessibilityStringForRangeParameterizedAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrangeforpositionparameterizedattribute?language=objc)
    pub static NSAccessibilityRangeForPositionParameterizedAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrangeforindexparameterizedattribute?language=objc)
    pub static NSAccessibilityRangeForIndexParameterizedAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityboundsforrangeparameterizedattribute?language=objc)
    pub static NSAccessibilityBoundsForRangeParameterizedAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrtfforrangeparameterizedattribute?language=objc)
    pub static NSAccessibilityRTFForRangeParameterizedAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitystylerangeforindexparameterizedattribute?language=objc)
    pub static NSAccessibilityStyleRangeForIndexParameterizedAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityattributedstringforrangeparameterizedattribute?language=objc)
    pub static NSAccessibilityAttributedStringForRangeParameterizedAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfonttextattribute?language=objc)
    pub static NSAccessibilityFontTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityforegroundcolortextattribute?language=objc)
    pub static NSAccessibilityForegroundColorTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitybackgroundcolortextattribute?language=objc)
    pub static NSAccessibilityBackgroundColorTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityunderlinecolortextattribute?language=objc)
    pub static NSAccessibilityUnderlineColorTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitystrikethroughcolortextattribute?language=objc)
    pub static NSAccessibilityStrikethroughColorTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityunderlinetextattribute?language=objc)
    pub static NSAccessibilityUnderlineTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysuperscripttextattribute?language=objc)
    pub static NSAccessibilitySuperscriptTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitystrikethroughtextattribute?language=objc)
    pub static NSAccessibilityStrikethroughTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityshadowtextattribute?language=objc)
    pub static NSAccessibilityShadowTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityattachmenttextattribute?language=objc)
    pub static NSAccessibilityAttachmentTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylinktextattribute?language=objc)
    pub static NSAccessibilityLinkTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityautocorrectedtextattribute?language=objc)
    pub static NSAccessibilityAutocorrectedTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytextalignmentattribute?language=objc)
    pub static NSAccessibilityTextAlignmentAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfontboldattribute?language=objc)
    pub static NSAccessibilityFontBoldAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfontitalicattribute?language=objc)
    pub static NSAccessibilityFontItalicAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitychildreninnavigationorderattribute?language=objc)
    pub static NSAccessibilityChildrenInNavigationOrderAttribute:
        &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityindexforchilduielementattribute?language=objc)
    pub static NSAccessibilityIndexForChildUIElementAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityindexforchilduielementinnavigationorderattribute?language=objc)
    pub static NSAccessibilityIndexForChildUIElementInNavigationOrderAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylistitemprefixtextattribute?language=objc)
    pub static NSAccessibilityListItemPrefixTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylistitemindextextattribute?language=objc)
    pub static NSAccessibilityListItemIndexTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylistitemleveltextattribute?language=objc)
    pub static NSAccessibilityListItemLevelTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymisspelledtextattribute?language=objc)
    pub static NSAccessibilityMisspelledTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymarkedmisspelledtextattribute?language=objc)
    pub static NSAccessibilityMarkedMisspelledTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylanguagetextattribute?language=objc)
    pub static NSAccessibilityLanguageTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycustomtextattribute?language=objc)
    pub static NSAccessibilityCustomTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityannotationtextattribute?language=objc)
    pub static NSAccessibilityAnnotationTextAttribute: &'static NSAttributedStringKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytextcompletionattribute?language=objc)
    pub static NSAccessibilityTextCompletionAttribute: &'static NSString;
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityannotationattributekey?language=objc)
// NS_TYPED_ENUM
pub type NSAccessibilityAnnotationAttributeKey = NSString;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityannotationlabel?language=objc)
    pub static NSAccessibilityAnnotationLabel: &'static NSAccessibilityAnnotationAttributeKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityannotationelement?language=objc)
    pub static NSAccessibilityAnnotationElement: &'static NSAccessibilityAnnotationAttributeKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityannotationlocation?language=objc)
    pub static NSAccessibilityAnnotationLocation: &'static NSAccessibilityAnnotationAttributeKey;
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityannotationposition?language=objc)
// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct NSAccessibilityAnnotationPosition(pub NSInteger);
impl NSAccessibilityAnnotationPosition {
    #[doc(alias = "NSAccessibilityAnnotationPositionFullRange")]
    pub const FullRange: Self = Self(0);
    #[doc(alias = "NSAccessibilityAnnotationPositionStart")]
    pub const Start: Self = Self(1);
    #[doc(alias = "NSAccessibilityAnnotationPositionEnd")]
    pub const End: Self = Self(2);
}

unsafe impl Encode for NSAccessibilityAnnotationPosition {
    const ENCODING: Encoding = NSInteger::ENCODING;
}

unsafe impl RefEncode for NSAccessibilityAnnotationPosition {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfontattributekey?language=objc)
// NS_TYPED_ENUM
pub type NSAccessibilityFontAttributeKey = NSString;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfontnamekey?language=objc)
    pub static NSAccessibilityFontNameKey: &'static NSAccessibilityFontAttributeKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfontfamilykey?language=objc)
    pub static NSAccessibilityFontFamilyKey: &'static NSAccessibilityFontAttributeKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityvisiblenamekey?language=objc)
    pub static NSAccessibilityVisibleNameKey: &'static NSAccessibilityFontAttributeKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfontsizekey?language=objc)
    pub static NSAccessibilityFontSizeKey: &'static NSAccessibilityFontAttributeKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymainattribute?language=objc)
    pub static NSAccessibilityMainAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityminimizedattribute?language=objc)
    pub static NSAccessibilityMinimizedAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityclosebuttonattribute?language=objc)
    pub static NSAccessibilityCloseButtonAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityzoombuttonattribute?language=objc)
    pub static NSAccessibilityZoomButtonAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityminimizebuttonattribute?language=objc)
    pub static NSAccessibilityMinimizeButtonAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytoolbarbuttonattribute?language=objc)
    pub static NSAccessibilityToolbarButtonAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityproxyattribute?language=objc)
    pub static NSAccessibilityProxyAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitygrowareaattribute?language=objc)
    pub static NSAccessibilityGrowAreaAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymodalattribute?language=objc)
    pub static NSAccessibilityModalAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydefaultbuttonattribute?language=objc)
    pub static NSAccessibilityDefaultButtonAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycancelbuttonattribute?language=objc)
    pub static NSAccessibilityCancelButtonAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfullscreenbuttonattribute?language=objc)
    pub static NSAccessibilityFullScreenButtonAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymenubarattribute?language=objc)
    pub static NSAccessibilityMenuBarAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitywindowsattribute?language=objc)
    pub static NSAccessibilityWindowsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfrontmostattribute?language=objc)
    pub static NSAccessibilityFrontmostAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityhiddenattribute?language=objc)
    pub static NSAccessibilityHiddenAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymainwindowattribute?language=objc)
    pub static NSAccessibilityMainWindowAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfocusedwindowattribute?language=objc)
    pub static NSAccessibilityFocusedWindowAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfocuseduielementattribute?language=objc)
    pub static NSAccessibilityFocusedUIElementAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityextrasmenubarattribute?language=objc)
    pub static NSAccessibilityExtrasMenuBarAttribute: &'static NSAccessibilityAttributeName;
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityorientation?language=objc)
// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct NSAccessibilityOrientation(pub NSInteger);
impl NSAccessibilityOrientation {
    #[doc(alias = "NSAccessibilityOrientationUnknown")]
    pub const Unknown: Self = Self(0);
    #[doc(alias = "NSAccessibilityOrientationVertical")]
    pub const Vertical: Self = Self(1);
    #[doc(alias = "NSAccessibilityOrientationHorizontal")]
    pub const Horizontal: Self = Self(2);
}

unsafe impl Encode for NSAccessibilityOrientation {
    const ENCODING: Encoding = NSInteger::ENCODING;
}

unsafe impl RefEncode for NSAccessibilityOrientation {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityorientationattribute?language=objc)
    pub static NSAccessibilityOrientationAttribute: &'static NSAccessibilityAttributeName;
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityorientationvalue?language=objc)
// NS_TYPED_ENUM
pub type NSAccessibilityOrientationValue = NSString;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityverticalorientationvalue?language=objc)
    pub static NSAccessibilityVerticalOrientationValue: &'static NSAccessibilityOrientationValue;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityhorizontalorientationvalue?language=objc)
    pub static NSAccessibilityHorizontalOrientationValue: &'static NSAccessibilityOrientationValue;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityunknownorientationvalue?language=objc)
    pub static NSAccessibilityUnknownOrientationValue: &'static NSAccessibilityOrientationValue;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycolumntitlesattribute?language=objc)
    pub static NSAccessibilityColumnTitlesAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysearchbuttonattribute?language=objc)
    pub static NSAccessibilitySearchButtonAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysearchmenuattribute?language=objc)
    pub static NSAccessibilitySearchMenuAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityclearbuttonattribute?language=objc)
    pub static NSAccessibilityClearButtonAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrowsattribute?language=objc)
    pub static NSAccessibilityRowsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityvisiblerowsattribute?language=objc)
    pub static NSAccessibilityVisibleRowsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityselectedrowsattribute?language=objc)
    pub static NSAccessibilitySelectedRowsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycolumnsattribute?language=objc)
    pub static NSAccessibilityColumnsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityvisiblecolumnsattribute?language=objc)
    pub static NSAccessibilityVisibleColumnsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityselectedcolumnsattribute?language=objc)
    pub static NSAccessibilitySelectedColumnsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysortdirectionattribute?language=objc)
    pub static NSAccessibilitySortDirectionAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityselectedcellsattribute?language=objc)
    pub static NSAccessibilitySelectedCellsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityvisiblecellsattribute?language=objc)
    pub static NSAccessibilityVisibleCellsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrowheaderuielementsattribute?language=objc)
    pub static NSAccessibilityRowHeaderUIElementsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycolumnheaderuielementsattribute?language=objc)
    pub static NSAccessibilityColumnHeaderUIElementsAttribute:
        &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycellforcolumnandrowparameterizedattribute?language=objc)
    pub static NSAccessibilityCellForColumnAndRowParameterizedAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrowindexrangeattribute?language=objc)
    pub static NSAccessibilityRowIndexRangeAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycolumnindexrangeattribute?language=objc)
    pub static NSAccessibilityColumnIndexRangeAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityhorizontalunitsattribute?language=objc)
    pub static NSAccessibilityHorizontalUnitsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityverticalunitsattribute?language=objc)
    pub static NSAccessibilityVerticalUnitsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityhorizontalunitdescriptionattribute?language=objc)
    pub static NSAccessibilityHorizontalUnitDescriptionAttribute:
        &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityverticalunitdescriptionattribute?language=objc)
    pub static NSAccessibilityVerticalUnitDescriptionAttribute:
        &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylayoutpointforscreenpointparameterizedattribute?language=objc)
    pub static NSAccessibilityLayoutPointForScreenPointParameterizedAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylayoutsizeforscreensizeparameterizedattribute?language=objc)
    pub static NSAccessibilityLayoutSizeForScreenSizeParameterizedAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityscreenpointforlayoutpointparameterizedattribute?language=objc)
    pub static NSAccessibilityScreenPointForLayoutPointParameterizedAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityscreensizeforlayoutsizeparameterizedattribute?language=objc)
    pub static NSAccessibilityScreenSizeForLayoutSizeParameterizedAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityhandlesattribute?language=objc)
    pub static NSAccessibilityHandlesAttribute: &'static NSAccessibilityAttributeName;
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysortdirectionvalue?language=objc)
// NS_TYPED_ENUM
pub type NSAccessibilitySortDirectionValue = NSString;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityascendingsortdirectionvalue?language=objc)
    pub static NSAccessibilityAscendingSortDirectionValue:
        &'static NSAccessibilitySortDirectionValue;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydescendingsortdirectionvalue?language=objc)
    pub static NSAccessibilityDescendingSortDirectionValue:
        &'static NSAccessibilitySortDirectionValue;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityunknownsortdirectionvalue?language=objc)
    pub static NSAccessibilityUnknownSortDirectionValue: &'static NSAccessibilitySortDirectionValue;
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysortdirection?language=objc)
// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct NSAccessibilitySortDirection(pub NSInteger);
impl NSAccessibilitySortDirection {
    #[doc(alias = "NSAccessibilitySortDirectionUnknown")]
    pub const Unknown: Self = Self(0);
    #[doc(alias = "NSAccessibilitySortDirectionAscending")]
    pub const Ascending: Self = Self(1);
    #[doc(alias = "NSAccessibilitySortDirectionDescending")]
    pub const Descending: Self = Self(2);
}

unsafe impl Encode for NSAccessibilitySortDirection {
    const ENCODING: Encoding = NSInteger::ENCODING;
}

unsafe impl RefEncode for NSAccessibilitySortDirection {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydisclosingattribute?language=objc)
    pub static NSAccessibilityDisclosingAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydisclosedrowsattribute?language=objc)
    pub static NSAccessibilityDisclosedRowsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydisclosedbyrowattribute?language=objc)
    pub static NSAccessibilityDisclosedByRowAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydisclosurelevelattribute?language=objc)
    pub static NSAccessibilityDisclosureLevelAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityallowedvaluesattribute?language=objc)
    pub static NSAccessibilityAllowedValuesAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylabeluielementsattribute?language=objc)
    pub static NSAccessibilityLabelUIElementsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylabelvalueattribute?language=objc)
    pub static NSAccessibilityLabelValueAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymatteholeattribute?language=objc)
    #[deprecated]
    pub static NSAccessibilityMatteHoleAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymattecontentuielementattribute?language=objc)
    #[deprecated]
    pub static NSAccessibilityMatteContentUIElementAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymarkeruielementsattribute?language=objc)
    pub static NSAccessibilityMarkerUIElementsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymarkervaluesattribute?language=objc)
    pub static NSAccessibilityMarkerValuesAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymarkergroupuielementattribute?language=objc)
    pub static NSAccessibilityMarkerGroupUIElementAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityunitsattribute?language=objc)
    pub static NSAccessibilityUnitsAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityunitdescriptionattribute?language=objc)
    pub static NSAccessibilityUnitDescriptionAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymarkertypeattribute?language=objc)
    pub static NSAccessibilityMarkerTypeAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymarkertypedescriptionattribute?language=objc)
    pub static NSAccessibilityMarkerTypeDescriptionAttribute: &'static NSAccessibilityAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityidentifierattribute?language=objc)
    pub static NSAccessibilityIdentifierAttribute: &'static NSAccessibilityAttributeName;
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrulermarkertypevalue?language=objc)
// NS_TYPED_ENUM
pub type NSAccessibilityRulerMarkerTypeValue = NSString;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylefttabstopmarkertypevalue?language=objc)
    pub static NSAccessibilityLeftTabStopMarkerTypeValue:
        &'static NSAccessibilityRulerMarkerTypeValue;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrighttabstopmarkertypevalue?language=objc)
    pub static NSAccessibilityRightTabStopMarkerTypeValue:
        &'static NSAccessibilityRulerMarkerTypeValue;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycentertabstopmarkertypevalue?language=objc)
    pub static NSAccessibilityCenterTabStopMarkerTypeValue:
        &'static NSAccessibilityRulerMarkerTypeValue;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydecimaltabstopmarkertypevalue?language=objc)
    pub static NSAccessibilityDecimalTabStopMarkerTypeValue:
        &'static NSAccessibilityRulerMarkerTypeValue;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityheadindentmarkertypevalue?language=objc)
    pub static NSAccessibilityHeadIndentMarkerTypeValue:
        &'static NSAccessibilityRulerMarkerTypeValue;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytailindentmarkertypevalue?language=objc)
    pub static NSAccessibilityTailIndentMarkerTypeValue:
        &'static NSAccessibilityRulerMarkerTypeValue;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfirstlineindentmarkertypevalue?language=objc)
    pub static NSAccessibilityFirstLineIndentMarkerTypeValue:
        &'static NSAccessibilityRulerMarkerTypeValue;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityunknownmarkertypevalue?language=objc)
    pub static NSAccessibilityUnknownMarkerTypeValue: &'static NSAccessibilityRulerMarkerTypeValue;
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrulermarkertype?language=objc)
// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct NSAccessibilityRulerMarkerType(pub NSInteger);
impl NSAccessibilityRulerMarkerType {
    #[doc(alias = "NSAccessibilityRulerMarkerTypeUnknown")]
    pub const Unknown: Self = Self(0);
    #[doc(alias = "NSAccessibilityRulerMarkerTypeTabStopLeft")]
    pub const TabStopLeft: Self = Self(1);
    #[doc(alias = "NSAccessibilityRulerMarkerTypeTabStopRight")]
    pub const TabStopRight: Self = Self(2);
    #[doc(alias = "NSAccessibilityRulerMarkerTypeTabStopCenter")]
    pub const TabStopCenter: Self = Self(3);
    #[doc(alias = "NSAccessibilityRulerMarkerTypeTabStopDecimal")]
    pub const TabStopDecimal: Self = Self(4);
    #[doc(alias = "NSAccessibilityRulerMarkerTypeIndentHead")]
    pub const IndentHead: Self = Self(5);
    #[doc(alias = "NSAccessibilityRulerMarkerTypeIndentTail")]
    pub const IndentTail: Self = Self(6);
    #[doc(alias = "NSAccessibilityRulerMarkerTypeIndentFirstLine")]
    pub const IndentFirstLine: Self = Self(7);
}

unsafe impl Encode for NSAccessibilityRulerMarkerType {
    const ENCODING: Encoding = NSInteger::ENCODING;
}

unsafe impl RefEncode for NSAccessibilityRulerMarkerType {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrulerunitvalue?language=objc)
// NS_TYPED_ENUM
pub type NSAccessibilityRulerUnitValue = NSString;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityinchesunitvalue?language=objc)
    pub static NSAccessibilityInchesUnitValue: &'static NSAccessibilityRulerUnitValue;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycentimetersunitvalue?language=objc)
    pub static NSAccessibilityCentimetersUnitValue: &'static NSAccessibilityRulerUnitValue;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitypointsunitvalue?language=objc)
    pub static NSAccessibilityPointsUnitValue: &'static NSAccessibilityRulerUnitValue;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitypicasunitvalue?language=objc)
    pub static NSAccessibilityPicasUnitValue: &'static NSAccessibilityRulerUnitValue;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityunknownunitvalue?language=objc)
    pub static NSAccessibilityUnknownUnitValue: &'static NSAccessibilityRulerUnitValue;
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityunits?language=objc)
// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct NSAccessibilityUnits(pub NSInteger);
impl NSAccessibilityUnits {
    #[doc(alias = "NSAccessibilityUnitsUnknown")]
    pub const Unknown: Self = Self(0);
    #[doc(alias = "NSAccessibilityUnitsInches")]
    pub const Inches: Self = Self(1);
    #[doc(alias = "NSAccessibilityUnitsCentimeters")]
    pub const Centimeters: Self = Self(2);
    #[doc(alias = "NSAccessibilityUnitsPoints")]
    pub const Points: Self = Self(3);
    #[doc(alias = "NSAccessibilityUnitsPicas")]
    pub const Picas: Self = Self(4);
}

unsafe impl Encode for NSAccessibilityUnits {
    const ENCODING: Encoding = NSInteger::ENCODING;
}

unsafe impl RefEncode for NSAccessibilityUnits {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityactionname?language=objc)
// NS_TYPED_EXTENSIBLE_ENUM
pub type NSAccessibilityActionName = NSString;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitypressaction?language=objc)
    pub static NSAccessibilityPressAction: &'static NSAccessibilityActionName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityincrementaction?language=objc)
    pub static NSAccessibilityIncrementAction: &'static NSAccessibilityActionName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydecrementaction?language=objc)
    pub static NSAccessibilityDecrementAction: &'static NSAccessibilityActionName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityconfirmaction?language=objc)
    pub static NSAccessibilityConfirmAction: &'static NSAccessibilityActionName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitypickaction?language=objc)
    pub static NSAccessibilityPickAction: &'static NSAccessibilityActionName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycancelaction?language=objc)
    pub static NSAccessibilityCancelAction: &'static NSAccessibilityActionName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityraiseaction?language=objc)
    pub static NSAccessibilityRaiseAction: &'static NSAccessibilityActionName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityshowmenuaction?language=objc)
    pub static NSAccessibilityShowMenuAction: &'static NSAccessibilityActionName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydeleteaction?language=objc)
    pub static NSAccessibilityDeleteAction: &'static NSAccessibilityActionName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityscrolltovisibleaction?language=objc)
    pub static NSAccessibilityScrollToVisibleAction: &'static NSAccessibilityActionName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityshowalternateuiaction?language=objc)
    pub static NSAccessibilityShowAlternateUIAction: &'static NSAccessibilityActionName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityshowdefaultuiaction?language=objc)
    pub static NSAccessibilityShowDefaultUIAction: &'static NSAccessibilityActionName;
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitynotificationname?language=objc)
// NS_TYPED_ENUM
pub type NSAccessibilityNotificationName = NSString;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymainwindowchangednotification?language=objc)
    pub static NSAccessibilityMainWindowChangedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfocusedwindowchangednotification?language=objc)
    pub static NSAccessibilityFocusedWindowChangedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfocuseduielementchangednotification?language=objc)
    pub static NSAccessibilityFocusedUIElementChangedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityapplicationactivatednotification?language=objc)
    pub static NSAccessibilityApplicationActivatedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityapplicationdeactivatednotification?language=objc)
    pub static NSAccessibilityApplicationDeactivatedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityapplicationhiddennotification?language=objc)
    pub static NSAccessibilityApplicationHiddenNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityapplicationshownnotification?language=objc)
    pub static NSAccessibilityApplicationShownNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitywindowcreatednotification?language=objc)
    pub static NSAccessibilityWindowCreatedNotification: &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitywindowmovednotification?language=objc)
    pub static NSAccessibilityWindowMovedNotification: &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitywindowresizednotification?language=objc)
    pub static NSAccessibilityWindowResizedNotification: &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitywindowminiaturizednotification?language=objc)
    pub static NSAccessibilityWindowMiniaturizedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitywindowdeminiaturizednotification?language=objc)
    pub static NSAccessibilityWindowDeminiaturizedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydrawercreatednotification?language=objc)
    pub static NSAccessibilityDrawerCreatedNotification: &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysheetcreatednotification?language=objc)
    pub static NSAccessibilitySheetCreatedNotification: &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityuielementdestroyednotification?language=objc)
    pub static NSAccessibilityUIElementDestroyedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityvaluechangednotification?language=objc)
    pub static NSAccessibilityValueChangedNotification: &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytitlechangednotification?language=objc)
    pub static NSAccessibilityTitleChangedNotification: &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityresizednotification?language=objc)
    pub static NSAccessibilityResizedNotification: &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymovednotification?language=objc)
    pub static NSAccessibilityMovedNotification: &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycreatednotification?language=objc)
    pub static NSAccessibilityCreatedNotification: &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylayoutchangednotification?language=objc)
    pub static NSAccessibilityLayoutChangedNotification: &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityhelptagcreatednotification?language=objc)
    pub static NSAccessibilityHelpTagCreatedNotification: &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityselectedtextchangednotification?language=objc)
    pub static NSAccessibilitySelectedTextChangedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrowcountchangednotification?language=objc)
    pub static NSAccessibilityRowCountChangedNotification: &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityselectedchildrenchangednotification?language=objc)
    pub static NSAccessibilitySelectedChildrenChangedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityselectedrowschangednotification?language=objc)
    pub static NSAccessibilitySelectedRowsChangedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityselectedcolumnschangednotification?language=objc)
    pub static NSAccessibilitySelectedColumnsChangedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrowexpandednotification?language=objc)
    pub static NSAccessibilityRowExpandedNotification: &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrowcollapsednotification?language=objc)
    pub static NSAccessibilityRowCollapsedNotification: &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityautocorrectionoccurrednotification?language=objc)
    pub static NSAccessibilityAutocorrectionOccurredNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytextinputmarkingsessionbegannotification?language=objc)
    pub static NSAccessibilityTextInputMarkingSessionBeganNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytextinputmarkingsessionendednotification?language=objc)
    pub static NSAccessibilityTextInputMarkingSessionEndedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydraggingsourcedragbegannotification?language=objc)
    pub static NSAccessibilityDraggingSourceDragBeganNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydraggingsourcedragendednotification?language=objc)
    pub static NSAccessibilityDraggingSourceDragEndedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydraggingdestinationdropallowednotification?language=objc)
    pub static NSAccessibilityDraggingDestinationDropAllowedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydraggingdestinationdropnotallowednotification?language=objc)
    pub static NSAccessibilityDraggingDestinationDropNotAllowedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydraggingdestinationdragacceptednotification?language=objc)
    pub static NSAccessibilityDraggingDestinationDragAcceptedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydraggingdestinationdragnotacceptednotification?language=objc)
    pub static NSAccessibilityDraggingDestinationDragNotAcceptedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityselectedcellschangednotification?language=objc)
    pub static NSAccessibilitySelectedCellsChangedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityunitschangednotification?language=objc)
    pub static NSAccessibilityUnitsChangedNotification: &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityselectedchildrenmovednotification?language=objc)
    pub static NSAccessibilitySelectedChildrenMovedNotification:
        &'static NSAccessibilityNotificationName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityannouncementrequestednotification?language=objc)
    pub static NSAccessibilityAnnouncementRequestedNotification:
        &'static NSAccessibilityNotificationName;
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrole?language=objc)
// NS_TYPED_ENUM
pub type NSAccessibilityRole = NSString;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityunknownrole?language=objc)
    pub static NSAccessibilityUnknownRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitybuttonrole?language=objc)
    pub static NSAccessibilityButtonRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityradiobuttonrole?language=objc)
    pub static NSAccessibilityRadioButtonRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycheckboxrole?language=objc)
    pub static NSAccessibilityCheckBoxRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysliderrole?language=objc)
    pub static NSAccessibilitySliderRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytabgrouprole?language=objc)
    pub static NSAccessibilityTabGroupRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytextfieldrole?language=objc)
    pub static NSAccessibilityTextFieldRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitystatictextrole?language=objc)
    pub static NSAccessibilityStaticTextRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytextarearole?language=objc)
    pub static NSAccessibilityTextAreaRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityscrollarearole?language=objc)
    pub static NSAccessibilityScrollAreaRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitypopupbuttonrole?language=objc)
    pub static NSAccessibilityPopUpButtonRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymenubuttonrole?language=objc)
    pub static NSAccessibilityMenuButtonRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytablerole?language=objc)
    pub static NSAccessibilityTableRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityapplicationrole?language=objc)
    pub static NSAccessibilityApplicationRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitygrouprole?language=objc)
    pub static NSAccessibilityGroupRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityradiogrouprole?language=objc)
    pub static NSAccessibilityRadioGroupRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylistrole?language=objc)
    pub static NSAccessibilityListRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityscrollbarrole?language=objc)
    pub static NSAccessibilityScrollBarRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityvalueindicatorrole?language=objc)
    pub static NSAccessibilityValueIndicatorRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityimagerole?language=objc)
    pub static NSAccessibilityImageRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymenubarrole?language=objc)
    pub static NSAccessibilityMenuBarRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymenubaritemrole?language=objc)
    pub static NSAccessibilityMenuBarItemRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymenurole?language=objc)
    pub static NSAccessibilityMenuRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymenuitemrole?language=objc)
    pub static NSAccessibilityMenuItemRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycolumnrole?language=objc)
    pub static NSAccessibilityColumnRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrowrole?language=objc)
    pub static NSAccessibilityRowRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytoolbarrole?language=objc)
    pub static NSAccessibilityToolbarRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitybusyindicatorrole?language=objc)
    pub static NSAccessibilityBusyIndicatorRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityprogressindicatorrole?language=objc)
    pub static NSAccessibilityProgressIndicatorRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitywindowrole?language=objc)
    pub static NSAccessibilityWindowRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydrawerrole?language=objc)
    pub static NSAccessibilityDrawerRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysystemwiderole?language=objc)
    pub static NSAccessibilitySystemWideRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityoutlinerole?language=objc)
    pub static NSAccessibilityOutlineRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityincrementorrole?language=objc)
    pub static NSAccessibilityIncrementorRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitybrowserrole?language=objc)
    pub static NSAccessibilityBrowserRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycomboboxrole?language=objc)
    pub static NSAccessibilityComboBoxRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysplitgrouprole?language=objc)
    pub static NSAccessibilitySplitGroupRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysplitterrole?language=objc)
    pub static NSAccessibilitySplitterRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycolorwellrole?language=objc)
    pub static NSAccessibilityColorWellRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitygrowarearole?language=objc)
    pub static NSAccessibilityGrowAreaRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysheetrole?language=objc)
    pub static NSAccessibilitySheetRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityhelptagrole?language=objc)
    pub static NSAccessibilityHelpTagRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymatterole?language=objc)
    pub static NSAccessibilityMatteRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrulerrole?language=objc)
    pub static NSAccessibilityRulerRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrulermarkerrole?language=objc)
    pub static NSAccessibilityRulerMarkerRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylinkrole?language=objc)
    pub static NSAccessibilityLinkRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydisclosuretrianglerole?language=objc)
    pub static NSAccessibilityDisclosureTriangleRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitygridrole?language=objc)
    pub static NSAccessibilityGridRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityrelevanceindicatorrole?language=objc)
    pub static NSAccessibilityRelevanceIndicatorRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydatetimearearole?language=objc)
    pub static NSAccessibilityDateTimeAreaRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylevelindicatorrole?language=objc)
    pub static NSAccessibilityLevelIndicatorRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycellrole?language=objc)
    pub static NSAccessibilityCellRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitypopoverrole?language=objc)
    pub static NSAccessibilityPopoverRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitypagerole?language=objc)
    pub static NSAccessibilityPageRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityheadingrole?language=objc)
    pub static NSAccessibilityHeadingRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylistmarkerrole?language=objc)
    pub static NSAccessibilityListMarkerRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitywebarearole?language=objc)
    pub static NSAccessibilityWebAreaRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylayoutarearole?language=objc)
    pub static NSAccessibilityLayoutAreaRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylayoutitemrole?language=objc)
    pub static NSAccessibilityLayoutItemRole: &'static NSAccessibilityRole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityhandlerole?language=objc)
    pub static NSAccessibilityHandleRole: &'static NSAccessibilityRole;
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysubrole?language=objc)
// NS_TYPED_ENUM
pub type NSAccessibilitySubrole = NSString;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityunknownsubrole?language=objc)
    pub static NSAccessibilityUnknownSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityclosebuttonsubrole?language=objc)
    pub static NSAccessibilityCloseButtonSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityzoombuttonsubrole?language=objc)
    pub static NSAccessibilityZoomButtonSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityminimizebuttonsubrole?language=objc)
    pub static NSAccessibilityMinimizeButtonSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytoolbarbuttonsubrole?language=objc)
    pub static NSAccessibilityToolbarButtonSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytablerowsubrole?language=objc)
    pub static NSAccessibilityTableRowSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityoutlinerowsubrole?language=objc)
    pub static NSAccessibilityOutlineRowSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysecuretextfieldsubrole?language=objc)
    pub static NSAccessibilitySecureTextFieldSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitystandardwindowsubrole?language=objc)
    pub static NSAccessibilityStandardWindowSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydialogsubrole?language=objc)
    pub static NSAccessibilityDialogSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysystemdialogsubrole?language=objc)
    pub static NSAccessibilitySystemDialogSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfloatingwindowsubrole?language=objc)
    pub static NSAccessibilityFloatingWindowSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysystemfloatingwindowsubrole?language=objc)
    pub static NSAccessibilitySystemFloatingWindowSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityincrementarrowsubrole?language=objc)
    pub static NSAccessibilityIncrementArrowSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydecrementarrowsubrole?language=objc)
    pub static NSAccessibilityDecrementArrowSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityincrementpagesubrole?language=objc)
    pub static NSAccessibilityIncrementPageSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydecrementpagesubrole?language=objc)
    pub static NSAccessibilityDecrementPageSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysearchfieldsubrole?language=objc)
    pub static NSAccessibilitySearchFieldSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytextattachmentsubrole?language=objc)
    pub static NSAccessibilityTextAttachmentSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytextlinksubrole?language=objc)
    pub static NSAccessibilityTextLinkSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytimelinesubrole?language=objc)
    pub static NSAccessibilityTimelineSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysortbuttonsubrole?language=objc)
    pub static NSAccessibilitySortButtonSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityratingindicatorsubrole?language=objc)
    pub static NSAccessibilityRatingIndicatorSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycontentlistsubrole?language=objc)
    pub static NSAccessibilityContentListSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydefinitionlistsubrole?language=objc)
    pub static NSAccessibilityDefinitionListSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfullscreenbuttonsubrole?language=objc)
    pub static NSAccessibilityFullScreenButtonSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytogglesubrole?language=objc)
    pub static NSAccessibilityToggleSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityswitchsubrole?language=objc)
    pub static NSAccessibilitySwitchSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydescriptionlistsubrole?language=objc)
    pub static NSAccessibilityDescriptionListSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytabbuttonsubrole?language=objc)
    pub static NSAccessibilityTabButtonSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycollectionlistsubrole?language=objc)
    pub static NSAccessibilityCollectionListSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysectionlistsubrole?language=objc)
    pub static NSAccessibilitySectionListSubrole: &'static NSAccessibilitySubrole;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysuggestionsubrole?language=objc)
    pub static NSAccessibilitySuggestionSubrole: &'static NSAccessibilitySubrole;
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitynotificationuserinfokey?language=objc)
// NS_TYPED_ENUM
pub type NSAccessibilityNotificationUserInfoKey = NSString;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityuielementskey?language=objc)
    pub static NSAccessibilityUIElementsKey: &'static NSAccessibilityNotificationUserInfoKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityprioritykey?language=objc)
    pub static NSAccessibilityPriorityKey: &'static NSAccessibilityNotificationUserInfoKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityannouncementkey?language=objc)
    pub static NSAccessibilityAnnouncementKey: &'static NSAccessibilityNotificationUserInfoKey;
}

extern "C-unwind" {
    /// # Safety
    ///
    /// - `element` should be of the correct type.
    /// - `user_info` generic should be of the correct type.
    pub fn NSAccessibilityPostNotificationWithUserInfo(
        element: &AnyObject,
        notification: &NSAccessibilityNotificationName,
        user_info: Option<&NSDictionary<NSAccessibilityNotificationUserInfoKey, AnyObject>>,
    );
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityprioritylevel?language=objc)
// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct NSAccessibilityPriorityLevel(pub NSInteger);
impl NSAccessibilityPriorityLevel {
    #[doc(alias = "NSAccessibilityPriorityLow")]
    pub const Low: Self = Self(10);
    #[doc(alias = "NSAccessibilityPriorityMedium")]
    pub const Medium: Self = Self(50);
    #[doc(alias = "NSAccessibilityPriorityHigh")]
    pub const High: Self = Self(90);
}

unsafe impl Encode for NSAccessibilityPriorityLevel {
    const ENCODING: Encoding = NSInteger::ENCODING;
}

unsafe impl RefEncode for NSAccessibilityPriorityLevel {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityloadingtoken?language=objc)
pub type NSAccessibilityLoadingToken = AnyObject;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityuielementsforsearchpredicateparameterizedattribute?language=objc)
    pub static NSAccessibilityUIElementsForSearchPredicateParameterizedAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityresultsforsearchpredicateparameterizedattribute?language=objc)
    pub static NSAccessibilityResultsForSearchPredicateParameterizedAttribute:
        &'static NSAccessibilityParameterizedAttributeName;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysearchidentifierskey?language=objc)
    pub static NSAccessibilitySearchIdentifiersKey: &'static NSString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysearchcurrentelementkey?language=objc)
    pub static NSAccessibilitySearchCurrentElementKey: &'static NSString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysearchcurrentrangekey?language=objc)
    pub static NSAccessibilitySearchCurrentRangeKey: &'static NSString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysearchdirectionkey?language=objc)
    pub static NSAccessibilitySearchDirectionKey: &'static NSString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysearchresultslimitkey?language=objc)
    pub static NSAccessibilitySearchResultsLimitKey: &'static NSString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysearchtextkey?language=objc)
    pub static NSAccessibilitySearchTextKey: &'static NSString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysearchdirectionnext?language=objc)
    pub static NSAccessibilitySearchDirectionNext: &'static NSString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysearchdirectionprevious?language=objc)
    pub static NSAccessibilitySearchDirectionPrevious: &'static NSString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysearchresultelementkey?language=objc)
    pub static NSAccessibilitySearchResultElementKey: &'static NSString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysearchresultrangekey?language=objc)
    pub static NSAccessibilitySearchResultRangeKey: &'static NSString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysearchresultdescriptionoverridekey?language=objc)
    pub static NSAccessibilitySearchResultDescriptionOverrideKey: &'static NSString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysearchresultloaderkey?language=objc)
    pub static NSAccessibilitySearchResultLoaderKey: &'static NSString;
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysearchkey?language=objc)
// NS_TYPED_ENUM
pub type NSAccessibilitySearchKey = NSString;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityanytypesearchkey?language=objc)
    pub static NSAccessibilityAnyTypeSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityarticlesearchkey?language=objc)
    pub static NSAccessibilityArticleSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityblockquotesamelevelsearchkey?language=objc)
    pub static NSAccessibilityBlockquoteSameLevelSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityblockquotesearchkey?language=objc)
    pub static NSAccessibilityBlockquoteSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityboldfontsearchkey?language=objc)
    pub static NSAccessibilityBoldFontSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitybuttonsearchkey?language=objc)
    pub static NSAccessibilityButtonSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycheckboxsearchkey?language=objc)
    pub static NSAccessibilityCheckBoxSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitycontrolsearchkey?language=objc)
    pub static NSAccessibilityControlSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydifferenttypesearchkey?language=objc)
    pub static NSAccessibilityDifferentTypeSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfontchangesearchkey?language=objc)
    pub static NSAccessibilityFontChangeSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityfontcolorchangesearchkey?language=objc)
    pub static NSAccessibilityFontColorChangeSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityframesearchkey?language=objc)
    pub static NSAccessibilityFrameSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitygraphicsearchkey?language=objc)
    pub static NSAccessibilityGraphicSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityheadinglevel1searchkey?language=objc)
    pub static NSAccessibilityHeadingLevel1SearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityheadinglevel2searchkey?language=objc)
    pub static NSAccessibilityHeadingLevel2SearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityheadinglevel3searchkey?language=objc)
    pub static NSAccessibilityHeadingLevel3SearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityheadinglevel4searchkey?language=objc)
    pub static NSAccessibilityHeadingLevel4SearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityheadinglevel5searchkey?language=objc)
    pub static NSAccessibilityHeadingLevel5SearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityheadinglevel6searchkey?language=objc)
    pub static NSAccessibilityHeadingLevel6SearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityheadingsamelevelsearchkey?language=objc)
    pub static NSAccessibilityHeadingSameLevelSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityheadingsearchkey?language=objc)
    pub static NSAccessibilityHeadingSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityitalicfontsearchkey?language=objc)
    pub static NSAccessibilityItalicFontSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitykeyboardfocusablesearchkey?language=objc)
    pub static NSAccessibilityKeyboardFocusableSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylandmarksearchkey?language=objc)
    pub static NSAccessibilityLandmarkSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylinksearchkey?language=objc)
    pub static NSAccessibilityLinkSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitylistsearchkey?language=objc)
    pub static NSAccessibilityListSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityliveregionsearchkey?language=objc)
    pub static NSAccessibilityLiveRegionSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitymisspelledwordsearchkey?language=objc)
    pub static NSAccessibilityMisspelledWordSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityoutlinesearchkey?language=objc)
    pub static NSAccessibilityOutlineSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityplaintextsearchkey?language=objc)
    pub static NSAccessibilityPlainTextSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityradiogroupsearchkey?language=objc)
    pub static NSAccessibilityRadioGroupSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysametypesearchkey?language=objc)
    pub static NSAccessibilitySameTypeSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitystatictextsearchkey?language=objc)
    pub static NSAccessibilityStaticTextSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitystylechangesearchkey?language=objc)
    pub static NSAccessibilityStyleChangeSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytablesamelevelsearchkey?language=objc)
    pub static NSAccessibilityTableSameLevelSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytablesearchkey?language=objc)
    pub static NSAccessibilityTableSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytextfieldsearchkey?language=objc)
    pub static NSAccessibilityTextFieldSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytextstatechangetypekey?language=objc)
    pub static NSAccessibilityTextStateChangeTypeKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitytextstatesynckey?language=objc)
    pub static NSAccessibilityTextStateSyncKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityunderlinesearchkey?language=objc)
    pub static NSAccessibilityUnderlineSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityunvisitedlinksearchkey?language=objc)
    pub static NSAccessibilityUnvisitedLinkSearchKey: &'static NSAccessibilitySearchKey;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityvisitedlinksearchkey?language=objc)
    pub static NSAccessibilityVisitedLinkSearchKey: &'static NSAccessibilitySearchKey;
}

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitydatetimecomponentsflags?language=objc)
pub type NSAccessibilityDateTimeComponentsFlags = NSUInteger;

/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityhourminutedatetimecomponentsflag?language=objc)
pub const NSAccessibilityHourMinuteDateTimeComponentsFlag: c_uint = 0x000c;
/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityhourminuteseconddatetimecomponentsflag?language=objc)
pub const NSAccessibilityHourMinuteSecondDateTimeComponentsFlag: c_uint = 0x000e;
/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityyearmonthdatetimecomponentsflag?language=objc)
pub const NSAccessibilityYearMonthDateTimeComponentsFlag: c_uint = 0x00c0;
/// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilityyearmonthdaydatetimecomponentsflag?language=objc)
pub const NSAccessibilityYearMonthDayDateTimeComponentsFlag: c_uint = 0x00e0;

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/appkit/nsaccessibilitysortbuttonrole?language=objc)
    #[deprecated]
    pub static NSAccessibilitySortButtonRole: &'static NSAccessibilityRole;
}
