//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct CLMonitorConfiguration;

    unsafe impl ClassType for CLMonitorConfiguration {
        type Super = NSObject;
        type Mutability = InteriorMutable;
    }
);

unsafe impl Send for CLMonitorConfiguration {}

unsafe impl Sync for CLMonitorConfiguration {}

unsafe impl NSObjectProtocol for CLMonitorConfiguration {}

extern_methods!(
    unsafe impl CLMonitorConfiguration {
        #[method_id(@__retain_semantics Other name)]
        pub unsafe fn name(&self) -> Retained<NSString>;

        #[cfg(all(
            feature = "CLMonitor",
            feature = "CLMonitoringEvent",
            feature = "block2"
        ))]
        #[method(eventHandler)]
        pub unsafe fn eventHandler(
            &self,
        ) -> NonNull<block2::Block<dyn Fn(NonNull<CLMonitor>, NonNull<CLMonitoringEvent>)>>;
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl CLMonitorConfiguration {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new() -> Retained<Self>;
    }
);
