//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_protocol!(
    pub unsafe trait NSAccessibilityColor {
        #[method_id(@__retain_semantics Other accessibilityName)]
        unsafe fn accessibilityName(&self) -> Retained<NSString>;
    }

    unsafe impl ProtocolType for dyn NSAccessibilityColor {}
);

extern_methods!(
    /// NSAccessibilityColorConformance
    #[cfg(feature = "NSColor")]
    unsafe impl NSColor {}
);

#[cfg(feature = "NSColor")]
unsafe impl NSAccessibilityColor for NSColor {}
