//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ffi::*;

use crate::*;

/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_4?language=objc)
pub const NSCoreDataVersionNumber10_4: c_float = 46.0;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_4_3?language=objc)
pub const NSCoreDataVersionNumber10_4_3: c_float = 77.0;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_5?language=objc)
pub const NSCoreDataVersionNumber10_5: c_float = 185.0;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_5_3?language=objc)
pub const NSCoreDataVersionNumber10_5_3: c_float = 186.0;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_6?language=objc)
pub const NSCoreDataVersionNumber10_6: c_float = 246.0;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_6_2?language=objc)
pub const NSCoreDataVersionNumber10_6_2: c_float = 250.0;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_6_3?language=objc)
pub const NSCoreDataVersionNumber10_6_3: c_float = 251.0;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_7?language=objc)
pub const NSCoreDataVersionNumber10_7: c_float = 358.4;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_7_2?language=objc)
pub const NSCoreDataVersionNumber10_7_2: c_float = 358.12;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_7_3?language=objc)
pub const NSCoreDataVersionNumber10_7_3: c_float = 358.13;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_7_4?language=objc)
pub const NSCoreDataVersionNumber10_7_4: c_float = 358.14;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_8?language=objc)
pub const NSCoreDataVersionNumber10_8: c_float = 407.5;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_8_2?language=objc)
pub const NSCoreDataVersionNumber10_8_2: c_float = 407.7;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_9?language=objc)
pub const NSCoreDataVersionNumber10_9: c_float = 481.0;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_9_2?language=objc)
pub const NSCoreDataVersionNumber10_9_2: c_float = 481.1;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_9_3?language=objc)
pub const NSCoreDataVersionNumber10_9_3: c_float = 481.3;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_10?language=objc)
pub const NSCoreDataVersionNumber10_10: c_float = 526.0;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_10_2?language=objc)
pub const NSCoreDataVersionNumber10_10_2: c_float = 526.1;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_10_3?language=objc)
pub const NSCoreDataVersionNumber10_10_3: c_float = 526.2;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_11?language=objc)
pub const NSCoreDataVersionNumber10_11: c_float = 640.0;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber10_11_3?language=objc)
pub const NSCoreDataVersionNumber10_11_3: c_float = 641.3;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_3_0?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_3_0: c_float = 241.0;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_3_1?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_3_1: c_float = 248.0;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_3_2?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_3_2: c_float = 310.2;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_4_0?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_4_0: c_float = 320.5;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_4_1?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_4_1: c_float = 320.11;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_4_2?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_4_2: c_float = 320.15;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_4_3?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_4_3: c_float = 320.17;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_5_0?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_5_0: c_float = 386.1;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_5_1?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_5_1: c_float = 386.5;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_6_0?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_6_0: c_float = 419.0;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_6_1?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_6_1: c_float = 419.1;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_7_0?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_7_0: c_float = 479.1;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_7_1?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_7_1: c_float = 479.3;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_8_0?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_8_0: c_float = 519.0;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_8_3?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_8_3: c_float = 519.15;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_9_0?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_9_0: c_float = 640.0;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_9_2?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_9_2: c_float = 641.4;
/// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber_iphoneos_9_3?language=objc)
pub const NSCoreDataVersionNumber_iPhoneOS_9_3: c_float = 641.6;
extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/coredata/nscoredataversionnumber?language=objc)
    pub static NSCoreDataVersionNumber: c_double;
}
