// This file has been automatically generated by `objc2`'s `header-translator`.
// DO NOT EDIT

#![allow(unused_imports)]
#![allow(deprecated)]
#![allow(non_snake_case)]
#![allow(non_camel_case_types)]
#![allow(non_upper_case_globals)]
#![allow(missing_docs)]
#![allow(clippy::too_many_arguments)]
#![allow(clippy::type_complexity)]
#![allow(clippy::upper_case_acronyms)]
#![allow(clippy::identity_op)]
#![allow(clippy::missing_safety_doc)]

#[link(name = "CoreData", kind = "framework")]
extern "C" {}

#[cfg(feature = "CoreDataDefines")]
#[path = "CoreDataDefines.rs"]
mod __CoreDataDefines;
#[cfg(feature = "CoreDataErrors")]
#[path = "CoreDataErrors.rs"]
mod __CoreDataErrors;
#[cfg(feature = "NSAtomicStore")]
#[path = "NSAtomicStore.rs"]
mod __NSAtomicStore;
#[cfg(feature = "NSAtomicStoreCacheNode")]
#[path = "NSAtomicStoreCacheNode.rs"]
mod __NSAtomicStoreCacheNode;
#[cfg(feature = "NSAttributeDescription")]
#[path = "NSAttributeDescription.rs"]
mod __NSAttributeDescription;
#[cfg(feature = "NSBatchDeleteRequest")]
#[path = "NSBatchDeleteRequest.rs"]
mod __NSBatchDeleteRequest;
#[cfg(feature = "NSBatchInsertRequest")]
#[path = "NSBatchInsertRequest.rs"]
mod __NSBatchInsertRequest;
#[cfg(feature = "NSBatchUpdateRequest")]
#[path = "NSBatchUpdateRequest.rs"]
mod __NSBatchUpdateRequest;
#[cfg(feature = "NSCompositeAttributeDescription")]
#[path = "NSCompositeAttributeDescription.rs"]
mod __NSCompositeAttributeDescription;
#[cfg(feature = "NSCoreDataCoreSpotlightDelegate")]
#[path = "NSCoreDataCoreSpotlightDelegate.rs"]
mod __NSCoreDataCoreSpotlightDelegate;
#[cfg(feature = "NSCustomMigrationStage")]
#[path = "NSCustomMigrationStage.rs"]
mod __NSCustomMigrationStage;
#[cfg(feature = "NSDerivedAttributeDescription")]
#[path = "NSDerivedAttributeDescription.rs"]
mod __NSDerivedAttributeDescription;
#[cfg(feature = "NSEntityDescription")]
#[path = "NSEntityDescription.rs"]
mod __NSEntityDescription;
#[cfg(feature = "NSEntityMapping")]
#[path = "NSEntityMapping.rs"]
mod __NSEntityMapping;
#[cfg(feature = "NSEntityMigrationPolicy")]
#[path = "NSEntityMigrationPolicy.rs"]
mod __NSEntityMigrationPolicy;
#[cfg(feature = "NSExpressionDescription")]
#[path = "NSExpressionDescription.rs"]
mod __NSExpressionDescription;
#[cfg(feature = "NSFetchIndexDescription")]
#[path = "NSFetchIndexDescription.rs"]
mod __NSFetchIndexDescription;
#[cfg(feature = "NSFetchIndexElementDescription")]
#[path = "NSFetchIndexElementDescription.rs"]
mod __NSFetchIndexElementDescription;
#[cfg(feature = "NSFetchRequest")]
#[path = "NSFetchRequest.rs"]
mod __NSFetchRequest;
#[cfg(feature = "NSFetchRequestExpression")]
#[path = "NSFetchRequestExpression.rs"]
mod __NSFetchRequestExpression;
#[cfg(feature = "NSFetchedPropertyDescription")]
#[path = "NSFetchedPropertyDescription.rs"]
mod __NSFetchedPropertyDescription;
#[cfg(feature = "NSFetchedResultsController")]
#[path = "NSFetchedResultsController.rs"]
mod __NSFetchedResultsController;
#[cfg(feature = "NSIncrementalStore")]
#[path = "NSIncrementalStore.rs"]
mod __NSIncrementalStore;
#[cfg(feature = "NSIncrementalStoreNode")]
#[path = "NSIncrementalStoreNode.rs"]
mod __NSIncrementalStoreNode;
#[cfg(feature = "NSLightweightMigrationStage")]
#[path = "NSLightweightMigrationStage.rs"]
mod __NSLightweightMigrationStage;
#[cfg(feature = "NSManagedObject")]
#[path = "NSManagedObject.rs"]
mod __NSManagedObject;
#[cfg(feature = "NSManagedObjectContext")]
#[path = "NSManagedObjectContext.rs"]
mod __NSManagedObjectContext;
#[cfg(feature = "NSManagedObjectID")]
#[path = "NSManagedObjectID.rs"]
mod __NSManagedObjectID;
#[cfg(feature = "NSManagedObjectModel")]
#[path = "NSManagedObjectModel.rs"]
mod __NSManagedObjectModel;
#[cfg(feature = "NSManagedObjectModelReference")]
#[path = "NSManagedObjectModelReference.rs"]
mod __NSManagedObjectModelReference;
#[cfg(feature = "NSMappingModel")]
#[path = "NSMappingModel.rs"]
mod __NSMappingModel;
#[cfg(feature = "NSMergePolicy")]
#[path = "NSMergePolicy.rs"]
mod __NSMergePolicy;
#[cfg(feature = "NSMigrationManager")]
#[path = "NSMigrationManager.rs"]
mod __NSMigrationManager;
#[cfg(feature = "NSMigrationStage")]
#[path = "NSMigrationStage.rs"]
mod __NSMigrationStage;
#[cfg(feature = "NSPersistentCloudKitContainer")]
#[path = "NSPersistentCloudKitContainer.rs"]
mod __NSPersistentCloudKitContainer;
#[cfg(feature = "NSPersistentCloudKitContainerEvent")]
#[path = "NSPersistentCloudKitContainerEvent.rs"]
mod __NSPersistentCloudKitContainerEvent;
#[cfg(feature = "NSPersistentCloudKitContainerEventRequest")]
#[path = "NSPersistentCloudKitContainerEventRequest.rs"]
mod __NSPersistentCloudKitContainerEventRequest;
#[cfg(feature = "NSPersistentCloudKitContainerOptions")]
#[path = "NSPersistentCloudKitContainerOptions.rs"]
mod __NSPersistentCloudKitContainerOptions;
#[cfg(feature = "NSPersistentContainer")]
#[path = "NSPersistentContainer.rs"]
mod __NSPersistentContainer;
#[cfg(feature = "NSPersistentHistoryChange")]
#[path = "NSPersistentHistoryChange.rs"]
mod __NSPersistentHistoryChange;
#[cfg(feature = "NSPersistentHistoryChangeRequest")]
#[path = "NSPersistentHistoryChangeRequest.rs"]
mod __NSPersistentHistoryChangeRequest;
#[cfg(feature = "NSPersistentHistoryToken")]
#[path = "NSPersistentHistoryToken.rs"]
mod __NSPersistentHistoryToken;
#[cfg(feature = "NSPersistentHistoryTransaction")]
#[path = "NSPersistentHistoryTransaction.rs"]
mod __NSPersistentHistoryTransaction;
#[cfg(feature = "NSPersistentStore")]
#[path = "NSPersistentStore.rs"]
mod __NSPersistentStore;
#[cfg(feature = "NSPersistentStoreCoordinator")]
#[path = "NSPersistentStoreCoordinator.rs"]
mod __NSPersistentStoreCoordinator;
#[cfg(feature = "NSPersistentStoreDescription")]
#[path = "NSPersistentStoreDescription.rs"]
mod __NSPersistentStoreDescription;
#[cfg(feature = "NSPersistentStoreRequest")]
#[path = "NSPersistentStoreRequest.rs"]
mod __NSPersistentStoreRequest;
#[cfg(feature = "NSPersistentStoreResult")]
#[path = "NSPersistentStoreResult.rs"]
mod __NSPersistentStoreResult;
#[cfg(feature = "NSPropertyDescription")]
#[path = "NSPropertyDescription.rs"]
mod __NSPropertyDescription;
#[cfg(feature = "NSPropertyMapping")]
#[path = "NSPropertyMapping.rs"]
mod __NSPropertyMapping;
#[cfg(feature = "NSQueryGenerationToken")]
#[path = "NSQueryGenerationToken.rs"]
mod __NSQueryGenerationToken;
#[cfg(feature = "NSRelationshipDescription")]
#[path = "NSRelationshipDescription.rs"]
mod __NSRelationshipDescription;
#[cfg(feature = "NSSaveChangesRequest")]
#[path = "NSSaveChangesRequest.rs"]
mod __NSSaveChangesRequest;
#[cfg(feature = "NSStagedMigrationManager")]
#[path = "NSStagedMigrationManager.rs"]
mod __NSStagedMigrationManager;

#[cfg(feature = "CoreDataDefines")]
pub use self::__CoreDataDefines::NSCoreDataVersionNumber;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSAffectedObjectsErrorKey;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSAffectedStoresErrorKey;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSCoreDataError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSDetailedErrorsKey;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSEntityMigrationPolicyError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSExternalRecordImportError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSInferredMappingModelError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSManagedObjectConstraintMergeError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSManagedObjectConstraintValidationError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSManagedObjectContextLockingError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSManagedObjectExternalRelationshipError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSManagedObjectMergeError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSManagedObjectModelReferenceNotFoundError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSManagedObjectReferentialIntegrityError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSManagedObjectValidationError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSMigrationCancelledError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSMigrationConstraintViolationError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSMigrationError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSMigrationManagerDestinationStoreError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSMigrationManagerSourceStoreError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSMigrationMissingMappingModelError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSMigrationMissingSourceModelError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSPersistentHistoryTokenExpiredError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSPersistentStoreCoordinatorLockingError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSPersistentStoreIncompatibleSchemaError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSPersistentStoreIncompatibleVersionHashError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSPersistentStoreIncompleteSaveError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSPersistentStoreInvalidTypeError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSPersistentStoreOpenError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSPersistentStoreOperationError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSPersistentStoreSaveConflictsError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSPersistentStoreSaveConflictsErrorKey;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSPersistentStoreSaveError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSPersistentStoreTimeoutError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSPersistentStoreTypeMismatchError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSPersistentStoreUnsupportedRequestTypeError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSSQLiteError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSSQLiteErrorDomain;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSStagedMigrationBackwardMigrationError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSStagedMigrationFrameworkVersionMismatchError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationDateTooLateError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationDateTooSoonError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationInvalidDateError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationInvalidURIError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationKeyErrorKey;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationMissingMandatoryPropertyError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationMultipleErrorsError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationNumberTooLargeError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationNumberTooSmallError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationObjectErrorKey;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationPredicateErrorKey;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationRelationshipDeniedDeleteError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationRelationshipExceedsMaximumCountError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationRelationshipLacksMinimumCountError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationStringPatternMatchingError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationStringTooLongError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationStringTooShortError;
#[cfg(feature = "CoreDataErrors")]
pub use self::__CoreDataErrors::NSValidationValueErrorKey;
#[cfg(all(feature = "NSAtomicStore", feature = "NSPersistentStore"))]
pub use self::__NSAtomicStore::NSAtomicStore;
#[cfg(feature = "NSAtomicStoreCacheNode")]
pub use self::__NSAtomicStoreCacheNode::NSAtomicStoreCacheNode;
#[cfg(all(feature = "NSAttributeDescription", feature = "NSPropertyDescription"))]
pub use self::__NSAttributeDescription::NSAttributeDescription;
#[cfg(feature = "NSAttributeDescription")]
pub use self::__NSAttributeDescription::NSAttributeType;
#[cfg(all(feature = "NSBatchDeleteRequest", feature = "NSPersistentStoreRequest"))]
pub use self::__NSBatchDeleteRequest::NSBatchDeleteRequest;
#[cfg(all(feature = "NSBatchInsertRequest", feature = "NSPersistentStoreRequest"))]
pub use self::__NSBatchInsertRequest::NSBatchInsertRequest;
#[cfg(all(feature = "NSBatchUpdateRequest", feature = "NSPersistentStoreRequest"))]
pub use self::__NSBatchUpdateRequest::NSBatchUpdateRequest;
#[cfg(all(
    feature = "NSAttributeDescription",
    feature = "NSCompositeAttributeDescription",
    feature = "NSPropertyDescription"
))]
pub use self::__NSCompositeAttributeDescription::NSCompositeAttributeDescription;
#[cfg(feature = "NSCoreDataCoreSpotlightDelegate")]
pub use self::__NSCoreDataCoreSpotlightDelegate::NSCoreDataCoreSpotlightDelegate;
#[cfg(feature = "NSCoreDataCoreSpotlightDelegate")]
pub use self::__NSCoreDataCoreSpotlightDelegate::NSCoreDataCoreSpotlightDelegateIndexDidUpdateNotification;
#[cfg(all(feature = "NSCustomMigrationStage", feature = "NSMigrationStage"))]
pub use self::__NSCustomMigrationStage::NSCustomMigrationStage;
#[cfg(all(
    feature = "NSAttributeDescription",
    feature = "NSDerivedAttributeDescription",
    feature = "NSPropertyDescription"
))]
pub use self::__NSDerivedAttributeDescription::NSDerivedAttributeDescription;
#[cfg(feature = "NSEntityDescription")]
pub use self::__NSEntityDescription::NSEntityDescription;
#[cfg(feature = "NSEntityMapping")]
pub use self::__NSEntityMapping::NSEntityMapping;
#[cfg(feature = "NSEntityMapping")]
pub use self::__NSEntityMapping::NSEntityMappingType;
#[cfg(feature = "NSEntityMigrationPolicy")]
pub use self::__NSEntityMigrationPolicy::NSEntityMigrationPolicy;
#[cfg(feature = "NSEntityMigrationPolicy")]
pub use self::__NSEntityMigrationPolicy::NSMigrationDestinationObjectKey;
#[cfg(feature = "NSEntityMigrationPolicy")]
pub use self::__NSEntityMigrationPolicy::NSMigrationEntityMappingKey;
#[cfg(feature = "NSEntityMigrationPolicy")]
pub use self::__NSEntityMigrationPolicy::NSMigrationEntityPolicyKey;
#[cfg(feature = "NSEntityMigrationPolicy")]
pub use self::__NSEntityMigrationPolicy::NSMigrationManagerKey;
#[cfg(feature = "NSEntityMigrationPolicy")]
pub use self::__NSEntityMigrationPolicy::NSMigrationPropertyMappingKey;
#[cfg(feature = "NSEntityMigrationPolicy")]
pub use self::__NSEntityMigrationPolicy::NSMigrationSourceObjectKey;
#[cfg(all(feature = "NSExpressionDescription", feature = "NSPropertyDescription"))]
pub use self::__NSExpressionDescription::NSExpressionDescription;
#[cfg(feature = "NSFetchIndexDescription")]
pub use self::__NSFetchIndexDescription::NSFetchIndexDescription;
#[cfg(feature = "NSFetchIndexElementDescription")]
pub use self::__NSFetchIndexElementDescription::NSFetchIndexElementDescription;
#[cfg(feature = "NSFetchIndexElementDescription")]
pub use self::__NSFetchIndexElementDescription::NSFetchIndexElementType;
#[cfg(all(feature = "NSFetchRequest", feature = "NSPersistentStoreRequest"))]
pub use self::__NSFetchRequest::NSAsynchronousFetchRequest;
#[cfg(all(feature = "NSFetchRequest", feature = "NSPersistentStoreRequest"))]
pub use self::__NSFetchRequest::NSFetchRequest;
#[cfg(feature = "NSFetchRequest")]
pub use self::__NSFetchRequest::NSFetchRequestResult;
#[cfg(feature = "NSFetchRequest")]
pub use self::__NSFetchRequest::NSFetchRequestResultType;
#[cfg(all(
    feature = "NSFetchRequest",
    feature = "NSPersistentStoreResult",
    feature = "block2"
))]
pub use self::__NSFetchRequest::NSPersistentStoreAsynchronousFetchResultCompletionBlock;
#[cfg(feature = "NSFetchRequestExpression")]
pub use self::__NSFetchRequestExpression::NSFetchRequestExpression;
#[cfg(feature = "NSFetchRequestExpression")]
pub use self::__NSFetchRequestExpression::NSFetchRequestExpressionType;
#[cfg(all(
    feature = "NSFetchedPropertyDescription",
    feature = "NSPropertyDescription"
))]
pub use self::__NSFetchedPropertyDescription::NSFetchedPropertyDescription;
#[cfg(feature = "NSFetchedResultsController")]
pub use self::__NSFetchedResultsController::NSFetchedResultsChangeType;
#[cfg(feature = "NSFetchedResultsController")]
pub use self::__NSFetchedResultsController::NSFetchedResultsController;
#[cfg(feature = "NSFetchedResultsController")]
pub use self::__NSFetchedResultsController::NSFetchedResultsControllerDelegate;
#[cfg(feature = "NSFetchedResultsController")]
pub use self::__NSFetchedResultsController::NSFetchedResultsSectionInfo;
#[cfg(all(feature = "NSIncrementalStore", feature = "NSPersistentStore"))]
pub use self::__NSIncrementalStore::NSIncrementalStore;
#[cfg(feature = "NSIncrementalStoreNode")]
pub use self::__NSIncrementalStoreNode::NSIncrementalStoreNode;
#[cfg(all(feature = "NSLightweightMigrationStage", feature = "NSMigrationStage"))]
pub use self::__NSLightweightMigrationStage::NSLightweightMigrationStage;
#[cfg(feature = "NSManagedObject")]
pub use self::__NSManagedObject::NSManagedObject;
#[cfg(feature = "NSManagedObject")]
pub use self::__NSManagedObject::NSSnapshotEventType;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSDeletedObjectIDsKey;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSDeletedObjectsKey;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSInsertedObjectIDsKey;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSInsertedObjectsKey;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSInvalidatedAllObjectsKey;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSInvalidatedObjectIDsKey;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSInvalidatedObjectsKey;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSManagedObjectContext;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSManagedObjectContextConcurrencyType;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSManagedObjectContextDidMergeChangesObjectIDsNotification;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSManagedObjectContextDidSaveNotification;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSManagedObjectContextDidSaveObjectIDsNotification;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSManagedObjectContextObjectsDidChangeNotification;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSManagedObjectContextQueryGenerationKey;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSManagedObjectContextWillSaveNotification;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSRefreshedObjectIDsKey;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSRefreshedObjectsKey;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSUpdatedObjectIDsKey;
#[cfg(feature = "NSManagedObjectContext")]
pub use self::__NSManagedObjectContext::NSUpdatedObjectsKey;
#[cfg(feature = "NSManagedObjectID")]
pub use self::__NSManagedObjectID::NSManagedObjectID;
#[cfg(feature = "NSManagedObjectModel")]
pub use self::__NSManagedObjectModel::NSManagedObjectModel;
#[cfg(feature = "NSManagedObjectModelReference")]
pub use self::__NSManagedObjectModelReference::NSManagedObjectModelReference;
#[cfg(feature = "NSMappingModel")]
pub use self::__NSMappingModel::NSMappingModel;
#[cfg(feature = "NSMergePolicy")]
pub use self::__NSMergePolicy::NSConstraintConflict;
#[cfg(feature = "NSMergePolicy")]
pub use self::__NSMergePolicy::NSMergeConflict;
#[cfg(feature = "NSMergePolicy")]
pub use self::__NSMergePolicy::NSMergePolicy;
#[cfg(feature = "NSMergePolicy")]
pub use self::__NSMergePolicy::NSMergePolicyType;
#[cfg(feature = "NSMigrationManager")]
pub use self::__NSMigrationManager::NSMigrationManager;
#[cfg(feature = "NSMigrationStage")]
pub use self::__NSMigrationStage::NSMigrationStage;
#[cfg(all(
    feature = "NSPersistentCloudKitContainer",
    feature = "NSPersistentContainer"
))]
pub use self::__NSPersistentCloudKitContainer::NSPersistentCloudKitContainer;
#[cfg(feature = "NSPersistentCloudKitContainer")]
pub use self::__NSPersistentCloudKitContainer::NSPersistentCloudKitContainerSchemaInitializationOptions;
#[cfg(feature = "NSPersistentCloudKitContainerEvent")]
pub use self::__NSPersistentCloudKitContainerEvent::NSPersistentCloudKitContainerEvent;
#[cfg(feature = "NSPersistentCloudKitContainerEvent")]
pub use self::__NSPersistentCloudKitContainerEvent::NSPersistentCloudKitContainerEventChangedNotification;
#[cfg(feature = "NSPersistentCloudKitContainerEvent")]
pub use self::__NSPersistentCloudKitContainerEvent::NSPersistentCloudKitContainerEventType;
#[cfg(feature = "NSPersistentCloudKitContainerEvent")]
pub use self::__NSPersistentCloudKitContainerEvent::NSPersistentCloudKitContainerEventUserInfoKey;
#[cfg(all(
    feature = "NSPersistentCloudKitContainerEventRequest",
    feature = "NSPersistentStoreRequest"
))]
pub use self::__NSPersistentCloudKitContainerEventRequest::NSPersistentCloudKitContainerEventRequest;
#[cfg(feature = "NSPersistentCloudKitContainerOptions")]
pub use self::__NSPersistentCloudKitContainerOptions::NSPersistentCloudKitContainerOptions;
#[cfg(feature = "NSPersistentContainer")]
pub use self::__NSPersistentContainer::NSPersistentContainer;
#[cfg(feature = "NSPersistentHistoryChange")]
pub use self::__NSPersistentHistoryChange::NSPersistentHistoryChange;
#[cfg(feature = "NSPersistentHistoryChange")]
pub use self::__NSPersistentHistoryChange::NSPersistentHistoryChangeType;
#[cfg(all(
    feature = "NSPersistentHistoryChangeRequest",
    feature = "NSPersistentStoreRequest"
))]
pub use self::__NSPersistentHistoryChangeRequest::NSPersistentHistoryChangeRequest;
#[cfg(feature = "NSPersistentHistoryToken")]
pub use self::__NSPersistentHistoryToken::NSPersistentHistoryToken;
#[cfg(feature = "NSPersistentHistoryTransaction")]
pub use self::__NSPersistentHistoryTransaction::NSPersistentHistoryTransaction;
#[cfg(feature = "NSPersistentStore")]
pub use self::__NSPersistentStore::NSPersistentStore;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSAddedPersistentStoresKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSBinaryExternalRecordType;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSBinaryStoreInsecureDecodingCompatibilityOption;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSBinaryStoreSecureDecodingClasses;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSBinaryStoreType;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSCoreDataCoreSpotlightExporter;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSEntityNameInPathKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSExternalRecordExtensionOption;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSExternalRecordsDirectoryOption;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSExternalRecordsFileFormatOption;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSIgnorePersistentStoreVersioningOption;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSInMemoryStoreType;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSInferMappingModelAutomaticallyOption;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSMigratePersistentStoresAutomaticallyOption;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSModelPathKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSObjectURIKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentHistoryTokenKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentHistoryTrackingKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreConnectionPoolMaxSizeKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreCoordinator;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreCoordinatorStoresDidChangeNotification;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreCoordinatorStoresWillChangeNotification;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreCoordinatorWillRemoveStoreNotification;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreDeferredLightweightMigrationOptionKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreDidImportUbiquitousContentChangesNotification;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreFileProtectionKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreForceDestroyOption;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreOSCompatibility;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreRebuildFromUbiquitousContentOption;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreRemoteChangeNotification;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreRemoteChangeNotificationPostOptionKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreRemoveUbiquitousMetadataOption;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreStagedMigrationManagerOptionKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreTimeoutOption;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreURLKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreUbiquitousContainerIdentifierKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreUbiquitousContentNameKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreUbiquitousContentURLKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreUbiquitousPeerTokenOption;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreUbiquitousTransitionType;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSPersistentStoreUbiquitousTransitionTypeKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSReadOnlyPersistentStoreOption;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSRemovedPersistentStoresKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSSQLiteAnalyzeOption;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSSQLiteManualVacuumOption;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSSQLitePragmasOption;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSSQLiteStoreType;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSStoreModelVersionHashesKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSStoreModelVersionIdentifiersKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSStorePathKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSStoreTypeKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSStoreUUIDInPathKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSStoreUUIDKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSUUIDChangedPersistentStoresKey;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSValidateXMLStoreOption;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSXMLExternalRecordType;
#[cfg(feature = "NSPersistentStoreCoordinator")]
pub use self::__NSPersistentStoreCoordinator::NSXMLStoreType;
#[cfg(feature = "NSPersistentStoreDescription")]
pub use self::__NSPersistentStoreDescription::NSPersistentStoreDescription;
#[cfg(feature = "NSPersistentStoreRequest")]
pub use self::__NSPersistentStoreRequest::NSPersistentStoreRequest;
#[cfg(feature = "NSPersistentStoreRequest")]
pub use self::__NSPersistentStoreRequest::NSPersistentStoreRequestType;
#[cfg(feature = "NSPersistentStoreResult")]
pub use self::__NSPersistentStoreResult::NSAsynchronousFetchResult;
#[cfg(feature = "NSPersistentStoreResult")]
pub use self::__NSPersistentStoreResult::NSBatchDeleteRequestResultType;
#[cfg(feature = "NSPersistentStoreResult")]
pub use self::__NSPersistentStoreResult::NSBatchDeleteResult;
#[cfg(feature = "NSPersistentStoreResult")]
pub use self::__NSPersistentStoreResult::NSBatchInsertRequestResultType;
#[cfg(feature = "NSPersistentStoreResult")]
pub use self::__NSPersistentStoreResult::NSBatchInsertResult;
#[cfg(feature = "NSPersistentStoreResult")]
pub use self::__NSPersistentStoreResult::NSBatchUpdateRequestResultType;
#[cfg(feature = "NSPersistentStoreResult")]
pub use self::__NSPersistentStoreResult::NSBatchUpdateResult;
#[cfg(feature = "NSPersistentStoreResult")]
pub use self::__NSPersistentStoreResult::NSPersistentCloudKitContainerEventResult;
#[cfg(feature = "NSPersistentStoreResult")]
pub use self::__NSPersistentStoreResult::NSPersistentCloudKitContainerEventResultType;
#[cfg(feature = "NSPersistentStoreResult")]
pub use self::__NSPersistentStoreResult::NSPersistentHistoryResult;
#[cfg(feature = "NSPersistentStoreResult")]
pub use self::__NSPersistentStoreResult::NSPersistentHistoryResultType;
#[cfg(feature = "NSPersistentStoreResult")]
pub use self::__NSPersistentStoreResult::NSPersistentStoreAsynchronousResult;
#[cfg(feature = "NSPersistentStoreResult")]
pub use self::__NSPersistentStoreResult::NSPersistentStoreResult;
#[cfg(feature = "NSPropertyDescription")]
pub use self::__NSPropertyDescription::NSPropertyDescription;
#[cfg(feature = "NSPropertyMapping")]
pub use self::__NSPropertyMapping::NSPropertyMapping;
#[cfg(feature = "NSQueryGenerationToken")]
pub use self::__NSQueryGenerationToken::NSQueryGenerationToken;
#[cfg(feature = "NSRelationshipDescription")]
pub use self::__NSRelationshipDescription::NSDeleteRule;
#[cfg(all(
    feature = "NSPropertyDescription",
    feature = "NSRelationshipDescription"
))]
pub use self::__NSRelationshipDescription::NSRelationshipDescription;
#[cfg(all(feature = "NSPersistentStoreRequest", feature = "NSSaveChangesRequest"))]
pub use self::__NSSaveChangesRequest::NSSaveChangesRequest;
#[cfg(feature = "NSStagedMigrationManager")]
pub use self::__NSStagedMigrationManager::NSStagedMigrationManager;
