//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ffi::*;
use objc2::__framework_prelude::*;

use crate::*;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_volume?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_Volume: AudioUnitParameterID = 7;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_sustain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_Sustain: AudioUnitParameterID = 64;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_sostenuto?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_Sostenuto: AudioUnitParameterID = 66;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_allnotesoff?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_AllNotesOff: AudioUnitParameterID = 123;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_modwheel?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_ModWheel: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_pitchbend?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_PitchBend: AudioUnitParameterID = 0xE0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_allsoundoff?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_AllSoundOff: AudioUnitParameterID = 120;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_resetallcontrollers?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_ResetAllControllers: AudioUnitParameterID = 121;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_pan?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_Pan: AudioUnitParameterID = 10;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_foot?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_Foot: AudioUnitParameterID = 4;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_channelpressure?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_ChannelPressure: AudioUnitParameterID = 0xD0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_keypressure?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_KeyPressure: AudioUnitParameterID = 0xA0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_expression?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_Expression: AudioUnitParameterID = 11;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_dataentry?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_DataEntry: AudioUnitParameterID = 6;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_volume_lsb?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_Volume_LSB: AudioUnitParameterID = kAUGroupParameterID_Volume + 32;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_modwheel_lsb?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_ModWheel_LSB: AudioUnitParameterID =
    kAUGroupParameterID_ModWheel + 32;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_pan_lsb?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_Pan_LSB: AudioUnitParameterID = kAUGroupParameterID_Pan + 32;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_foot_lsb?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_Foot_LSB: AudioUnitParameterID = kAUGroupParameterID_Foot + 32;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_expression_lsb?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_Expression_LSB: AudioUnitParameterID =
    kAUGroupParameterID_Expression + 32;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_dataentry_lsb?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_DataEntry_LSB: AudioUnitParameterID =
    kAUGroupParameterID_DataEntry + 32;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_keypressure_firstkey?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_KeyPressure_FirstKey: AudioUnitParameterID = 256;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaugroupparameterid_keypressure_lastkey?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUGroupParameterID_KeyPressure_LastKey: AudioUnitParameterID = 383;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kpannerparam_gain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kPannerParam_Gain: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kpannerparam_azimuth?language=objc)
#[cfg(feature = "AUComponent")]
pub const kPannerParam_Azimuth: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kpannerparam_elevation?language=objc)
#[cfg(feature = "AUComponent")]
pub const kPannerParam_Elevation: AudioUnitParameterID = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kpannerparam_distance?language=objc)
#[cfg(feature = "AUComponent")]
pub const kPannerParam_Distance: AudioUnitParameterID = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kpannerparam_coordscale?language=objc)
#[cfg(feature = "AUComponent")]
pub const kPannerParam_CoordScale: AudioUnitParameterID = 4;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kpannerparam_refdistance?language=objc)
#[cfg(feature = "AUComponent")]
pub const kPannerParam_RefDistance: AudioUnitParameterID = 5;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kspatialmixerparam_azimuth?language=objc)
#[cfg(feature = "AUComponent")]
pub const kSpatialMixerParam_Azimuth: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kspatialmixerparam_elevation?language=objc)
#[cfg(feature = "AUComponent")]
pub const kSpatialMixerParam_Elevation: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kspatialmixerparam_distance?language=objc)
#[cfg(feature = "AUComponent")]
pub const kSpatialMixerParam_Distance: AudioUnitParameterID = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kspatialmixerparam_gain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kSpatialMixerParam_Gain: AudioUnitParameterID = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kspatialmixerparam_playbackrate?language=objc)
#[cfg(feature = "AUComponent")]
pub const kSpatialMixerParam_PlaybackRate: AudioUnitParameterID = 4;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kspatialmixerparam_enable?language=objc)
#[cfg(feature = "AUComponent")]
pub const kSpatialMixerParam_Enable: AudioUnitParameterID = 5;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kspatialmixerparam_mingain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kSpatialMixerParam_MinGain: AudioUnitParameterID = 6;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kspatialmixerparam_maxgain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kSpatialMixerParam_MaxGain: AudioUnitParameterID = 7;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kspatialmixerparam_reverbblend?language=objc)
#[cfg(feature = "AUComponent")]
pub const kSpatialMixerParam_ReverbBlend: AudioUnitParameterID = 8;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kspatialmixerparam_globalreverbgain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kSpatialMixerParam_GlobalReverbGain: AudioUnitParameterID = 9;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kspatialmixerparam_occlusionattenuation?language=objc)
#[cfg(feature = "AUComponent")]
pub const kSpatialMixerParam_OcclusionAttenuation: AudioUnitParameterID = 10;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kspatialmixerparam_obstructionattenuation?language=objc)
#[cfg(feature = "AUComponent")]
pub const kSpatialMixerParam_ObstructionAttenuation: AudioUnitParameterID = 11;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kspatialmixerparam_headyaw?language=objc)
#[cfg(feature = "AUComponent")]
pub const kSpatialMixerParam_HeadYaw: AudioUnitParameterID = 19;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kspatialmixerparam_headpitch?language=objc)
#[cfg(feature = "AUComponent")]
pub const kSpatialMixerParam_HeadPitch: AudioUnitParameterID = 20;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kspatialmixerparam_headroll?language=objc)
#[cfg(feature = "AUComponent")]
pub const kSpatialMixerParam_HeadRoll: AudioUnitParameterID = 21;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_filterfrequency?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_FilterFrequency: AudioUnitParameterID = 14;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_filterbandwidth?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_FilterBandwidth: AudioUnitParameterID = 15;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_filtergain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_FilterGain: AudioUnitParameterID = 16;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_filtertype?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_FilterType: AudioUnitParameterID = 17;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_filterenable?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_FilterEnable: AudioUnitParameterID = 18;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_azimuth?language=objc)
#[cfg(feature = "AUComponent")]
pub const k3DMixerParam_Azimuth: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_elevation?language=objc)
#[cfg(feature = "AUComponent")]
pub const k3DMixerParam_Elevation: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_distance?language=objc)
#[cfg(feature = "AUComponent")]
pub const k3DMixerParam_Distance: AudioUnitParameterID = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_gain?language=objc)
#[cfg(feature = "AUComponent")]
pub const k3DMixerParam_Gain: AudioUnitParameterID = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_playbackrate?language=objc)
#[cfg(feature = "AUComponent")]
pub const k3DMixerParam_PlaybackRate: AudioUnitParameterID = 4;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_busenable?language=objc)
#[cfg(feature = "AUComponent")]
pub const k3DMixerParam_BusEnable: AudioUnitParameterID = 20;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_mingainindecibels?language=objc)
#[cfg(feature = "AUComponent")]
pub const k3DMixerParam_MinGainInDecibels: AudioUnitParameterID = 21;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_maxgainindecibels?language=objc)
#[cfg(feature = "AUComponent")]
pub const k3DMixerParam_MaxGainInDecibels: AudioUnitParameterID = 22;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_drywetreverbblend?language=objc)
#[cfg(feature = "AUComponent")]
pub const k3DMixerParam_DryWetReverbBlend: AudioUnitParameterID = 23;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_globalreverbgainindecibels?language=objc)
#[cfg(feature = "AUComponent")]
pub const k3DMixerParam_GlobalReverbGainInDecibels: AudioUnitParameterID = 24;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_occlusionattenuationindecibels?language=objc)
#[cfg(feature = "AUComponent")]
pub const k3DMixerParam_OcclusionAttenuationInDecibels: AudioUnitParameterID = 25;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_obstructionattenuationindecibels?language=objc)
#[cfg(feature = "AUComponent")]
pub const k3DMixerParam_ObstructionAttenuationInDecibels: AudioUnitParameterID = 26;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_reverbblend?language=objc)
#[cfg(feature = "AUComponent")]
#[deprecated]
pub const k3DMixerParam_ReverbBlend: AudioUnitParameterID = 5;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_globalreverbgain?language=objc)
#[cfg(feature = "AUComponent")]
#[deprecated]
pub const k3DMixerParam_GlobalReverbGain: AudioUnitParameterID = 6;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_occlusionattenuation?language=objc)
#[cfg(feature = "AUComponent")]
#[deprecated]
pub const k3DMixerParam_OcclusionAttenuation: AudioUnitParameterID = 7;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_obstructionattenuation?language=objc)
#[cfg(feature = "AUComponent")]
#[deprecated]
pub const k3DMixerParam_ObstructionAttenuation: AudioUnitParameterID = 8;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_mingain?language=objc)
#[cfg(feature = "AUComponent")]
#[deprecated]
pub const k3DMixerParam_MinGain: AudioUnitParameterID = 9;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_maxgain?language=objc)
#[cfg(feature = "AUComponent")]
#[deprecated]
pub const k3DMixerParam_MaxGain: AudioUnitParameterID = 10;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_preaveragepower?language=objc)
#[cfg(feature = "AUComponent")]
pub const k3DMixerParam_PreAveragePower: AudioUnitParameterID = 1000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_prepeakholdlevel?language=objc)
#[cfg(feature = "AUComponent")]
pub const k3DMixerParam_PrePeakHoldLevel: AudioUnitParameterID = 2000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_postaveragepower?language=objc)
#[cfg(feature = "AUComponent")]
pub const k3DMixerParam_PostAveragePower: AudioUnitParameterID = 3000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/k3dmixerparam_postpeakholdlevel?language=objc)
#[cfg(feature = "AUComponent")]
pub const k3DMixerParam_PostPeakHoldLevel: AudioUnitParameterID = 4000;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultichannelmixerparam_volume?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultiChannelMixerParam_Volume: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultichannelmixerparam_enable?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultiChannelMixerParam_Enable: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultichannelmixerparam_pan?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultiChannelMixerParam_Pan: AudioUnitParameterID = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultichannelmixerparam_preaveragepower?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultiChannelMixerParam_PreAveragePower: AudioUnitParameterID = 1000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultichannelmixerparam_prepeakholdlevel?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultiChannelMixerParam_PrePeakHoldLevel: AudioUnitParameterID = 2000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultichannelmixerparam_postaveragepower?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultiChannelMixerParam_PostAveragePower: AudioUnitParameterID = 3000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultichannelmixerparam_postpeakholdlevel?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultiChannelMixerParam_PostPeakHoldLevel: AudioUnitParameterID = 4000;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmatrixmixerparam_volume?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMatrixMixerParam_Volume: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmatrixmixerparam_enable?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMatrixMixerParam_Enable: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmatrixmixerparam_preaveragepower?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMatrixMixerParam_PreAveragePower: AudioUnitParameterID = 1000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmatrixmixerparam_prepeakholdlevel?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMatrixMixerParam_PrePeakHoldLevel: AudioUnitParameterID = 2000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmatrixmixerparam_postaveragepower?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMatrixMixerParam_PostAveragePower: AudioUnitParameterID = 3000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmatrixmixerparam_postpeakholdlevel?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMatrixMixerParam_PostPeakHoldLevel: AudioUnitParameterID = 4000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmatrixmixerparam_preaveragepowerlinear?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMatrixMixerParam_PreAveragePowerLinear: AudioUnitParameterID = 5000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmatrixmixerparam_prepeakholdlevellinear?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMatrixMixerParam_PrePeakHoldLevelLinear: AudioUnitParameterID = 6000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmatrixmixerparam_postaveragepowerlinear?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMatrixMixerParam_PostAveragePowerLinear: AudioUnitParameterID = 7000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmatrixmixerparam_postpeakholdlevellinear?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMatrixMixerParam_PostPeakHoldLevelLinear: AudioUnitParameterID = 8000;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/khaloutputparam_volume?language=objc)
#[cfg(feature = "AUComponent")]
pub const kHALOutputParam_Volume: AudioUnitParameterID = 14;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/ktimepitchparam_rate?language=objc)
#[cfg(feature = "AUComponent")]
pub const kTimePitchParam_Rate: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/ktimepitchparam_pitch?language=objc)
#[cfg(feature = "AUComponent")]
pub const kTimePitchParam_Pitch: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/ktimepitchparam_effectblend?language=objc)
#[cfg(feature = "AUComponent")]
pub const kTimePitchParam_EffectBlend: AudioUnitParameterID = 2;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/knewtimepitchparam_rate?language=objc)
#[cfg(feature = "AUComponent")]
pub const kNewTimePitchParam_Rate: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/knewtimepitchparam_pitch?language=objc)
#[cfg(feature = "AUComponent")]
pub const kNewTimePitchParam_Pitch: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/knewtimepitchparam_smoothness?language=objc)
#[cfg(feature = "AUComponent")]
pub const kNewTimePitchParam_Smoothness: AudioUnitParameterID = 4;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/knewtimepitchparam_overlap?language=objc)
#[cfg(feature = "AUComponent")]
#[deprecated]
pub const kNewTimePitchParam_Overlap: AudioUnitParameterID = kNewTimePitchParam_Smoothness;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/knewtimepitchparam_enablespectralcoherence?language=objc)
#[cfg(feature = "AUComponent")]
pub const kNewTimePitchParam_EnableSpectralCoherence: AudioUnitParameterID = 6;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/knewtimepitchparam_enablepeaklocking?language=objc)
#[cfg(feature = "AUComponent")]
#[deprecated]
pub const kNewTimePitchParam_EnablePeakLocking: AudioUnitParameterID =
    kNewTimePitchParam_EnableSpectralCoherence;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/knewtimepitchparam_enabletransientpreservation?language=objc)
#[cfg(feature = "AUComponent")]
pub const kNewTimePitchParam_EnableTransientPreservation: AudioUnitParameterID = 7;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kausamplerparam_gain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUSamplerParam_Gain: AudioUnitParameterID = 900;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kausamplerparam_coarsetuning?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUSamplerParam_CoarseTuning: AudioUnitParameterID = 901;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kausamplerparam_finetuning?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUSamplerParam_FineTuning: AudioUnitParameterID = 902;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kausamplerparam_pan?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUSamplerParam_Pan: AudioUnitParameterID = 903;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kbandpassparam_centerfrequency?language=objc)
#[cfg(feature = "AUComponent")]
pub const kBandpassParam_CenterFrequency: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kbandpassparam_bandwidth?language=objc)
#[cfg(feature = "AUComponent")]
pub const kBandpassParam_Bandwidth: AudioUnitParameterID = 1;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/khipassparam_cutofffrequency?language=objc)
#[cfg(feature = "AUComponent")]
pub const kHipassParam_CutoffFrequency: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/khipassparam_resonance?language=objc)
#[cfg(feature = "AUComponent")]
pub const kHipassParam_Resonance: AudioUnitParameterID = 1;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/klowpassparam_cutofffrequency?language=objc)
#[cfg(feature = "AUComponent")]
pub const kLowPassParam_CutoffFrequency: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/klowpassparam_resonance?language=objc)
#[cfg(feature = "AUComponent")]
pub const kLowPassParam_Resonance: AudioUnitParameterID = 1;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/khighshelfparam_cutofffrequency?language=objc)
#[cfg(feature = "AUComponent")]
pub const kHighShelfParam_CutOffFrequency: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/khighshelfparam_gain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kHighShelfParam_Gain: AudioUnitParameterID = 1;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaulowshelfparam_cutofffrequency?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAULowShelfParam_CutoffFrequency: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaulowshelfparam_gain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAULowShelfParam_Gain: AudioUnitParameterID = 1;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kparametriceqparam_centerfreq?language=objc)
#[cfg(feature = "AUComponent")]
pub const kParametricEQParam_CenterFreq: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kparametriceqparam_q?language=objc)
#[cfg(feature = "AUComponent")]
pub const kParametricEQParam_Q: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kparametriceqparam_gain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kParametricEQParam_Gain: AudioUnitParameterID = 2;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/klimiterparam_attacktime?language=objc)
#[cfg(feature = "AUComponent")]
pub const kLimiterParam_AttackTime: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/klimiterparam_decaytime?language=objc)
#[cfg(feature = "AUComponent")]
pub const kLimiterParam_DecayTime: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/klimiterparam_pregain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kLimiterParam_PreGain: AudioUnitParameterID = 2;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdynamicsprocessorparam_threshold?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDynamicsProcessorParam_Threshold: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdynamicsprocessorparam_headroom?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDynamicsProcessorParam_HeadRoom: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdynamicsprocessorparam_expansionratio?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDynamicsProcessorParam_ExpansionRatio: AudioUnitParameterID = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdynamicsprocessorparam_expansionthreshold?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDynamicsProcessorParam_ExpansionThreshold: AudioUnitParameterID = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdynamicsprocessorparam_attacktime?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDynamicsProcessorParam_AttackTime: AudioUnitParameterID = 4;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdynamicsprocessorparam_releasetime?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDynamicsProcessorParam_ReleaseTime: AudioUnitParameterID = 5;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdynamicsprocessorparam_overallgain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDynamicsProcessorParam_OverallGain: AudioUnitParameterID = 6;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdynamicsprocessorparam_mastergain?language=objc)
#[cfg(feature = "AUComponent")]
#[deprecated]
pub const kDynamicsProcessorParam_MasterGain: AudioUnitParameterID =
    kDynamicsProcessorParam_OverallGain;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdynamicsprocessorparam_compressionamount?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDynamicsProcessorParam_CompressionAmount: AudioUnitParameterID = 1000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdynamicsprocessorparam_inputamplitude?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDynamicsProcessorParam_InputAmplitude: AudioUnitParameterID = 2000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdynamicsprocessorparam_outputamplitude?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDynamicsProcessorParam_OutputAmplitude: AudioUnitParameterID = 3000;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kvarispeedparam_playbackrate?language=objc)
#[cfg(feature = "AUComponent")]
pub const kVarispeedParam_PlaybackRate: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kvarispeedparam_playbackcents?language=objc)
#[cfg(feature = "AUComponent")]
pub const kVarispeedParam_PlaybackCents: AudioUnitParameterID = 1;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdistortionparam_delay?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDistortionParam_Delay: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdistortionparam_decay?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDistortionParam_Decay: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdistortionparam_delaymix?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDistortionParam_DelayMix: AudioUnitParameterID = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdistortionparam_decimation?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDistortionParam_Decimation: AudioUnitParameterID = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdistortionparam_rounding?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDistortionParam_Rounding: AudioUnitParameterID = 4;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdistortionparam_decimationmix?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDistortionParam_DecimationMix: AudioUnitParameterID = 5;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdistortionparam_linearterm?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDistortionParam_LinearTerm: AudioUnitParameterID = 6;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdistortionparam_squaredterm?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDistortionParam_SquaredTerm: AudioUnitParameterID = 7;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdistortionparam_cubicterm?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDistortionParam_CubicTerm: AudioUnitParameterID = 8;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdistortionparam_polynomialmix?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDistortionParam_PolynomialMix: AudioUnitParameterID = 9;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdistortionparam_ringmodfreq1?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDistortionParam_RingModFreq1: AudioUnitParameterID = 10;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdistortionparam_ringmodfreq2?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDistortionParam_RingModFreq2: AudioUnitParameterID = 11;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdistortionparam_ringmodbalance?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDistortionParam_RingModBalance: AudioUnitParameterID = 12;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdistortionparam_ringmodmix?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDistortionParam_RingModMix: AudioUnitParameterID = 13;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdistortionparam_softclipgain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDistortionParam_SoftClipGain: AudioUnitParameterID = 14;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdistortionparam_finalmix?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDistortionParam_FinalMix: AudioUnitParameterID = 15;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdelayparam_wetdrymix?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDelayParam_WetDryMix: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdelayparam_delaytime?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDelayParam_DelayTime: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdelayparam_feedback?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDelayParam_Feedback: AudioUnitParameterID = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kdelayparam_lopasscutoff?language=objc)
#[cfg(feature = "AUComponent")]
pub const kDelayParam_LopassCutoff: AudioUnitParameterID = 3;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/ksampledelayparam_delayframes?language=objc)
#[cfg(feature = "AUComponent")]
pub const kSampleDelayParam_DelayFrames: AudioUnitParameterID = 0;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunbandeqparam_globalgain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUNBandEQParam_GlobalGain: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunbandeqparam_bypassband?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUNBandEQParam_BypassBand: AudioUnitParameterID = 1000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunbandeqparam_filtertype?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUNBandEQParam_FilterType: AudioUnitParameterID = 2000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunbandeqparam_frequency?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUNBandEQParam_Frequency: AudioUnitParameterID = 3000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunbandeqparam_gain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUNBandEQParam_Gain: AudioUnitParameterID = 4000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunbandeqparam_bandwidth?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUNBandEQParam_Bandwidth: AudioUnitParameterID = 5000;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunbandeqfiltertype_parametric?language=objc)
pub const kAUNBandEQFilterType_Parametric: c_long = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunbandeqfiltertype_2ndorderbutterworthlowpass?language=objc)
pub const kAUNBandEQFilterType_2ndOrderButterworthLowPass: c_long = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunbandeqfiltertype_2ndorderbutterworthhighpass?language=objc)
pub const kAUNBandEQFilterType_2ndOrderButterworthHighPass: c_long = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunbandeqfiltertype_resonantlowpass?language=objc)
pub const kAUNBandEQFilterType_ResonantLowPass: c_long = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunbandeqfiltertype_resonanthighpass?language=objc)
pub const kAUNBandEQFilterType_ResonantHighPass: c_long = 4;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunbandeqfiltertype_bandpass?language=objc)
pub const kAUNBandEQFilterType_BandPass: c_long = 5;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunbandeqfiltertype_bandstop?language=objc)
pub const kAUNBandEQFilterType_BandStop: c_long = 6;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunbandeqfiltertype_lowshelf?language=objc)
pub const kAUNBandEQFilterType_LowShelf: c_long = 7;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunbandeqfiltertype_highshelf?language=objc)
pub const kAUNBandEQFilterType_HighShelf: c_long = 8;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunbandeqfiltertype_resonantlowshelf?language=objc)
pub const kAUNBandEQFilterType_ResonantLowShelf: c_long = 9;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunbandeqfiltertype_resonanthighshelf?language=objc)
pub const kAUNBandEQFilterType_ResonantHighShelf: c_long = 10;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/knumaunbandeqfiltertypes?language=objc)
pub const kNumAUNBandEQFilterTypes: c_long = 11;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kroundtripaacparam_format?language=objc)
#[cfg(feature = "AUComponent")]
pub const kRoundTripAACParam_Format: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kroundtripaacparam_encodingstrategy?language=objc)
#[cfg(feature = "AUComponent")]
pub const kRoundTripAACParam_EncodingStrategy: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kroundtripaacparam_rateorquality?language=objc)
#[cfg(feature = "AUComponent")]
pub const kRoundTripAACParam_RateOrQuality: AudioUnitParameterID = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kroundtripaacparam_bitrate?language=objc)
#[cfg(feature = "AUComponent")]
pub const kRoundTripAACParam_BitRate: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kroundtripaacparam_quality?language=objc)
#[cfg(feature = "AUComponent")]
pub const kRoundTripAACParam_Quality: AudioUnitParameterID = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kroundtripaacparam_compressedformatsamplerate?language=objc)
#[cfg(feature = "AUComponent")]
pub const kRoundTripAACParam_CompressedFormatSampleRate: AudioUnitParameterID = 3;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kausoundisolationparam_wetdrymixpercent?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUSoundIsolationParam_WetDryMixPercent: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kausoundisolationparam_soundtoisolate?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUSoundIsolationParam_SoundToIsolate: AudioUnitParameterID = 1;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kausoundisolationsoundtype_highqualityvoice?language=objc)
pub const kAUSoundIsolationSoundType_HighQualityVoice: c_long = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kausoundisolationsoundtype_voice?language=objc)
pub const kAUSoundIsolationSoundType_Voice: c_long = 1;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kauaudiomixparameter_style?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUAudioMixParameter_Style: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kauaudiomixparameter_remixamount?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUAudioMixParameter_RemixAmount: AudioUnitParameterID = 1;

/// Constants available as values for parameter kAUAudioMixParameter_Style.
///
///
/// Cinematic rendering style (default)
///
///
/// Studio rendering style
///
///
/// In-Frame rendering style
///
///
/// Cinematic rendering style - background only
///
///
/// Cinematic rendering style - foreground only
///
///
/// Studio rendering style - foreground only
///
///
/// In-Frame rendering style - foreground only
///
///
/// Standard rendering style
///
///
/// Studio rendering style - background only
///
///
/// In-Frame rendering style - background only
///
/// See also [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/auaudiomixrenderingstyle?language=objc)
// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct AUAudioMixRenderingStyle(pub u32);
impl AUAudioMixRenderingStyle {
    #[doc(alias = "kAudioMixRenderingStyle_Cinematic")]
    pub const AudioMixRenderingStyle_Cinematic: Self = Self(0);
    #[doc(alias = "kAudioMixRenderingStyle_Studio")]
    pub const AudioMixRenderingStyle_Studio: Self = Self(1);
    #[doc(alias = "kAudioMixRenderingStyle_InFrame")]
    pub const AudioMixRenderingStyle_InFrame: Self = Self(2);
    #[doc(alias = "kAudioMixRenderingStyle_CinematicBackgroundStem")]
    pub const AudioMixRenderingStyle_CinematicBackgroundStem: Self = Self(3);
    #[doc(alias = "kAudioMixRenderingStyle_CinematicForegroundStem")]
    pub const AudioMixRenderingStyle_CinematicForegroundStem: Self = Self(4);
    #[doc(alias = "kAudioMixRenderingStyle_StudioForegroundStem")]
    pub const AudioMixRenderingStyle_StudioForegroundStem: Self = Self(5);
    #[doc(alias = "kAudioMixRenderingStyle_InFrameForegroundStem")]
    pub const AudioMixRenderingStyle_InFrameForegroundStem: Self = Self(6);
    #[doc(alias = "kAudioMixRenderingStyle_Standard")]
    pub const AudioMixRenderingStyle_Standard: Self = Self(7);
    #[doc(alias = "kAudioMixRenderingStyle_StudioBackgroundStem")]
    pub const AudioMixRenderingStyle_StudioBackgroundStem: Self = Self(8);
    #[doc(alias = "kAudioMixRenderingStyle_InFrameBackgroundStem")]
    pub const AudioMixRenderingStyle_InFrameBackgroundStem: Self = Self(9);
}

unsafe impl Encode for AUAudioMixRenderingStyle {
    const ENCODING: Encoding = u32::ENCODING;
}

unsafe impl RefEncode for AUAudioMixRenderingStyle {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kgraphiceqparam_numberofbands?language=objc)
#[cfg(feature = "AUComponent")]
pub const kGraphicEQParam_NumberOfBands: AudioUnitParameterID = 10000;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_drywetmix?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_DryWetMix: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_smalllargemix?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_SmallLargeMix: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_smallsize?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_SmallSize: AudioUnitParameterID = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_largesize?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_LargeSize: AudioUnitParameterID = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_predelay?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_PreDelay: AudioUnitParameterID = 4;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_largedelay?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_LargeDelay: AudioUnitParameterID = 5;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_smalldensity?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_SmallDensity: AudioUnitParameterID = 6;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_largedensity?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_LargeDensity: AudioUnitParameterID = 7;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_largedelayrange?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_LargeDelayRange: AudioUnitParameterID = 8;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_smallbrightness?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_SmallBrightness: AudioUnitParameterID = 9;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_largebrightness?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_LargeBrightness: AudioUnitParameterID = 10;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_smalldelayrange?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_SmallDelayRange: AudioUnitParameterID = 11;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_modulationrate?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_ModulationRate: AudioUnitParameterID = 12;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverbparam_modulationdepth?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverbParam_ModulationDepth: AudioUnitParameterID = 13;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_pregain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_Pregain: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_postgain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_Postgain: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_crossover1?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_Crossover1: AudioUnitParameterID = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_crossover2?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_Crossover2: AudioUnitParameterID = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_crossover3?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_Crossover3: AudioUnitParameterID = 4;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_threshold1?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_Threshold1: AudioUnitParameterID = 5;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_threshold2?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_Threshold2: AudioUnitParameterID = 6;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_threshold3?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_Threshold3: AudioUnitParameterID = 7;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_threshold4?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_Threshold4: AudioUnitParameterID = 8;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_headroom1?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_Headroom1: AudioUnitParameterID = 9;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_headroom2?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_Headroom2: AudioUnitParameterID = 10;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_headroom3?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_Headroom3: AudioUnitParameterID = 11;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_headroom4?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_Headroom4: AudioUnitParameterID = 12;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_attacktime?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_AttackTime: AudioUnitParameterID = 13;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_releasetime?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_ReleaseTime: AudioUnitParameterID = 14;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_eq1?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_EQ1: AudioUnitParameterID = 15;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_eq2?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_EQ2: AudioUnitParameterID = 16;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_eq3?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_EQ3: AudioUnitParameterID = 17;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_eq4?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_EQ4: AudioUnitParameterID = 18;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_compressionamount1?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_CompressionAmount1: AudioUnitParameterID = 1000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_compressionamount2?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_CompressionAmount2: AudioUnitParameterID = 2000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_compressionamount3?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_CompressionAmount3: AudioUnitParameterID = 3000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_compressionamount4?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_CompressionAmount4: AudioUnitParameterID = 4000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_inputamplitude1?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_InputAmplitude1: AudioUnitParameterID = 5000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_inputamplitude2?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_InputAmplitude2: AudioUnitParameterID = 6000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_inputamplitude3?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_InputAmplitude3: AudioUnitParameterID = 7000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_inputamplitude4?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_InputAmplitude4: AudioUnitParameterID = 8000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_outputamplitude1?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_OutputAmplitude1: AudioUnitParameterID = 9000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_outputamplitude2?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_OutputAmplitude2: AudioUnitParameterID = 10000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_outputamplitude3?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_OutputAmplitude3: AudioUnitParameterID = 11000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandcompressorparam_outputamplitude4?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandCompressorParam_OutputAmplitude4: AudioUnitParameterID = 12000;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandfilter_lowfiltertype?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandFilter_LowFilterType: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandfilter_lowfrequency?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandFilter_LowFrequency: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandfilter_lowgain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandFilter_LowGain: AudioUnitParameterID = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandfilter_centerfreq1?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandFilter_CenterFreq1: AudioUnitParameterID = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandfilter_centergain1?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandFilter_CenterGain1: AudioUnitParameterID = 4;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandfilter_bandwidth1?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandFilter_Bandwidth1: AudioUnitParameterID = 5;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandfilter_centerfreq2?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandFilter_CenterFreq2: AudioUnitParameterID = 6;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandfilter_centergain2?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandFilter_CenterGain2: AudioUnitParameterID = 7;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandfilter_bandwidth2?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandFilter_Bandwidth2: AudioUnitParameterID = 8;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandfilter_centerfreq3?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandFilter_CenterFreq3: AudioUnitParameterID = 9;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandfilter_centergain3?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandFilter_CenterGain3: AudioUnitParameterID = 10;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandfilter_bandwidth3?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandFilter_Bandwidth3: AudioUnitParameterID = 11;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandfilter_highfiltertype?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandFilter_HighFilterType: AudioUnitParameterID = 12;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandfilter_highfrequency?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandFilter_HighFrequency: AudioUnitParameterID = 13;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmultibandfilter_highgain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMultibandFilter_HighGain: AudioUnitParameterID = 14;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/krogerbeepparam_ingatethreshold?language=objc)
#[cfg(feature = "AUComponent")]
pub const kRogerBeepParam_InGateThreshold: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/krogerbeepparam_ingatethresholdtime?language=objc)
#[cfg(feature = "AUComponent")]
pub const kRogerBeepParam_InGateThresholdTime: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/krogerbeepparam_outgatethreshold?language=objc)
#[cfg(feature = "AUComponent")]
pub const kRogerBeepParam_OutGateThreshold: AudioUnitParameterID = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/krogerbeepparam_outgatethresholdtime?language=objc)
#[cfg(feature = "AUComponent")]
pub const kRogerBeepParam_OutGateThresholdTime: AudioUnitParameterID = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/krogerbeepparam_sensitivity?language=objc)
#[cfg(feature = "AUComponent")]
pub const kRogerBeepParam_Sensitivity: AudioUnitParameterID = 4;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/krogerbeepparam_rogertype?language=objc)
#[cfg(feature = "AUComponent")]
pub const kRogerBeepParam_RogerType: AudioUnitParameterID = 5;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/krogerbeepparam_rogergain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kRogerBeepParam_RogerGain: AudioUnitParameterID = 6;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kstereomixerparam_volume?language=objc)
#[cfg(feature = "AUComponent")]
pub const kStereoMixerParam_Volume: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kstereomixerparam_pan?language=objc)
#[cfg(feature = "AUComponent")]
pub const kStereoMixerParam_Pan: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kstereomixerparam_preaveragepower?language=objc)
#[cfg(feature = "AUComponent")]
pub const kStereoMixerParam_PreAveragePower: AudioUnitParameterID = 1000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kstereomixerparam_prepeakholdlevel?language=objc)
#[cfg(feature = "AUComponent")]
pub const kStereoMixerParam_PrePeakHoldLevel: AudioUnitParameterID = 2000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kstereomixerparam_postaveragepower?language=objc)
#[cfg(feature = "AUComponent")]
pub const kStereoMixerParam_PostAveragePower: AudioUnitParameterID = 3000;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kstereomixerparam_postpeakholdlevel?language=objc)
#[cfg(feature = "AUComponent")]
pub const kStereoMixerParam_PostPeakHoldLevel: AudioUnitParameterID = 4000;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunetreceiveparam_status?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUNetReceiveParam_Status: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunetreceiveparam_numparameters?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUNetReceiveParam_NumParameters: AudioUnitParameterID = 1;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunetsendparam_status?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUNetSendParam_Status: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunetsendparam_numparameters?language=objc)
#[cfg(feature = "AUComponent")]
pub const kAUNetSendParam_NumParameters: AudioUnitParameterID = 1;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunetstatus_notconnected?language=objc)
pub const kAUNetStatus_NotConnected: c_uint = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunetstatus_connected?language=objc)
pub const kAUNetStatus_Connected: c_uint = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunetstatus_overflow?language=objc)
pub const kAUNetStatus_Overflow: c_uint = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunetstatus_underflow?language=objc)
pub const kAUNetStatus_Underflow: c_uint = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunetstatus_connecting?language=objc)
pub const kAUNetStatus_Connecting: c_uint = 4;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kaunetstatus_listening?language=objc)
pub const kAUNetStatus_Listening: c_uint = 5;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmusicdeviceparam_tuning?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMusicDeviceParam_Tuning: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmusicdeviceparam_volume?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMusicDeviceParam_Volume: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kmusicdeviceparam_reverbvolume?language=objc)
#[cfg(feature = "AUComponent")]
pub const kMusicDeviceParam_ReverbVolume: AudioUnitParameterID = 2;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/krandomparam_bounda?language=objc)
#[cfg(feature = "AUComponent")]
pub const kRandomParam_BoundA: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/krandomparam_boundb?language=objc)
#[cfg(feature = "AUComponent")]
pub const kRandomParam_BoundB: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/krandomparam_curve?language=objc)
#[cfg(feature = "AUComponent")]
pub const kRandomParam_Curve: AudioUnitParameterID = 2;

/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverb2param_drywetmix?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverb2Param_DryWetMix: AudioUnitParameterID = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverb2param_gain?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverb2Param_Gain: AudioUnitParameterID = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverb2param_mindelaytime?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverb2Param_MinDelayTime: AudioUnitParameterID = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverb2param_maxdelaytime?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverb2Param_MaxDelayTime: AudioUnitParameterID = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverb2param_decaytimeat0hz?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverb2Param_DecayTimeAt0Hz: AudioUnitParameterID = 4;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverb2param_decaytimeatnyquist?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverb2Param_DecayTimeAtNyquist: AudioUnitParameterID = 5;
/// [Apple's documentation](https://developer.apple.com/documentation/audiotoolbox/kreverb2param_randomizereflections?language=objc)
#[cfg(feature = "AUComponent")]
pub const kReverb2Param_RandomizeReflections: AudioUnitParameterID = 6;
