//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;

use crate::*;

/// Controls how strongly the tint color applies in a view.
///
/// A tint prominence of `primary` will use the app's tint color with full
/// saturation and brightness, while `secondary` will apply it more subtly.
/// Using `none` gives an untinted appearance.
///
/// See also [Apple's documentation](https://developer.apple.com/documentation/appkit/nstintprominence?language=objc)
// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct NSTintProminence(pub NSInteger);
impl NSTintProminence {
    #[doc(alias = "NSTintProminenceAutomatic")]
    pub const Automatic: Self = Self(0);
    #[doc(alias = "NSTintProminenceNone")]
    pub const None: Self = Self(1);
    #[doc(alias = "NSTintProminencePrimary")]
    pub const Primary: Self = Self(2);
    #[doc(alias = "NSTintProminenceSecondary")]
    pub const Secondary: Self = Self(3);
}

unsafe impl Encode for NSTintProminence {
    const ENCODING: Encoding = NSInteger::ENCODING;
}

unsafe impl RefEncode for NSTintProminence {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}
