//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ffi::*;
use core::ptr::NonNull;
use objc2::__framework_prelude::*;
#[cfg(feature = "objc2-audio-toolbox")]
#[cfg(not(target_os = "watchos"))]
use objc2_audio_toolbox::*;

use crate::*;

extern_class!(
    /// an AVAudioUnitTimeEffect that provides good quality time stretching and pitch shifting
    ///
    /// In this time effect, the playback rate and pitch parameters function independently of each other
    ///
    /// See also [Apple's documentation](https://developer.apple.com/documentation/avfaudio/avaudiounittimepitch?language=objc)
    #[unsafe(super(AVAudioUnitTimeEffect, AVAudioUnit, AVAudioNode, NSObject))]
    #[derive(Debug, PartialEq, Eq, Hash)]
    #[cfg(all(
        feature = "AVAudioNode",
        feature = "AVAudioUnit",
        feature = "AVAudioUnitTimeEffect"
    ))]
    pub struct AVAudioUnitTimePitch;
);

#[cfg(all(
    feature = "AVAudioNode",
    feature = "AVAudioUnit",
    feature = "AVAudioUnitTimeEffect"
))]
extern_conformance!(
    unsafe impl NSObjectProtocol for AVAudioUnitTimePitch {}
);

#[cfg(all(
    feature = "AVAudioNode",
    feature = "AVAudioUnit",
    feature = "AVAudioUnitTimeEffect"
))]
impl AVAudioUnitTimePitch {
    extern_methods!(
        /// playback rate of the input signal
        ///
        /// Range:      1/32 -> 32.0
        /// Default:    1.0
        /// Unit:       Generic
        #[unsafe(method(rate))]
        #[unsafe(method_family = none)]
        pub unsafe fn rate(&self) -> c_float;

        /// Setter for [`rate`][Self::rate].
        #[unsafe(method(setRate:))]
        #[unsafe(method_family = none)]
        pub unsafe fn setRate(&self, rate: c_float);

        /// amount by which the input signal is pitch shifted
        ///
        /// 1 octave  = 1200 cents
        /// 1 musical semitone  = 100 cents
        ///
        /// Range:      -2400 -> 2400
        /// Default:    0.0
        /// Unit:       Cents
        #[unsafe(method(pitch))]
        #[unsafe(method_family = none)]
        pub unsafe fn pitch(&self) -> c_float;

        /// Setter for [`pitch`][Self::pitch].
        #[unsafe(method(setPitch:))]
        #[unsafe(method_family = none)]
        pub unsafe fn setPitch(&self, pitch: c_float);

        /// amount of overlap between segments of the input audio signal
        ///
        /// A higher value results in fewer artifacts in the output signal.
        /// This parameter also impacts the amount of CPU used.
        ///
        /// Range:      3.0 -> 32.0
        /// Default:    8.0
        /// Unit:       Generic
        #[unsafe(method(overlap))]
        #[unsafe(method_family = none)]
        pub unsafe fn overlap(&self) -> c_float;

        /// Setter for [`overlap`][Self::overlap].
        #[unsafe(method(setOverlap:))]
        #[unsafe(method_family = none)]
        pub unsafe fn setOverlap(&self, overlap: c_float);
    );
}

/// Methods declared on superclass `AVAudioUnitTimeEffect`.
#[cfg(all(
    feature = "AVAudioNode",
    feature = "AVAudioUnit",
    feature = "AVAudioUnitTimeEffect"
))]
impl AVAudioUnitTimePitch {
    extern_methods!(
        #[cfg(feature = "objc2-audio-toolbox")]
        #[cfg(not(target_os = "watchos"))]
        /// create an AVAudioUnitTimeEffect object
        ///
        ///
        /// Parameter `audioComponentDescription`: AudioComponentDescription of the audio unit to be initialized
        ///
        /// The componentType must be kAudioUnitType_FormatConverter
        #[unsafe(method(initWithAudioComponentDescription:))]
        #[unsafe(method_family = init)]
        pub unsafe fn initWithAudioComponentDescription(
            this: Allocated<Self>,
            audio_component_description: AudioComponentDescription,
        ) -> Retained<Self>;
    );
}

/// Methods declared on superclass `NSObject`.
#[cfg(all(
    feature = "AVAudioNode",
    feature = "AVAudioUnit",
    feature = "AVAudioUnitTimeEffect"
))]
impl AVAudioUnitTimePitch {
    extern_methods!(
        #[unsafe(method(init))]
        #[unsafe(method_family = init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[unsafe(method(new))]
        #[unsafe(method_family = new)]
        pub unsafe fn new() -> Retained<Self>;
    );
}
