//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

// NS_TYPED_EXTENSIBLE_ENUM
pub type NSUserInterfaceItemIdentifier = NSString;

extern_protocol!(
    pub unsafe trait NSUserInterfaceItemIdentification {
        #[method_id(@__retain_semantics Other identifier)]
        unsafe fn identifier(&self) -> Option<Retained<NSUserInterfaceItemIdentifier>>;

        #[method(setIdentifier:)]
        unsafe fn setIdentifier(&self, identifier: Option<&NSUserInterfaceItemIdentifier>);
    }

    unsafe impl ProtocolType for dyn NSUserInterfaceItemIdentification {}
);
