//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

// NS_OPTIONS
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct NSFontAssetRequestOptions(pub NSUInteger);
bitflags::bitflags! {
    impl NSFontAssetRequestOptions: NSUInteger {
        const NSFontAssetRequestOptionUsesStandardUI = 1<<0;
    }
}

unsafe impl Encode for NSFontAssetRequestOptions {
    const ENCODING: Encoding = NSUInteger::ENCODING;
}

unsafe impl RefEncode for NSFontAssetRequestOptions {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct NSFontAssetRequest;

    unsafe impl ClassType for NSFontAssetRequest {
        type Super = NSObject;
        type Mutability = InteriorMutable;
    }
);

unsafe impl NSObjectProtocol for NSFontAssetRequest {}

unsafe impl NSProgressReporting for NSFontAssetRequest {}

extern_methods!(
    unsafe impl NSFontAssetRequest {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[cfg(feature = "NSFontDescriptor")]
        #[method_id(@__retain_semantics Init initWithFontDescriptors:options:)]
        pub unsafe fn initWithFontDescriptors_options(
            this: Allocated<Self>,
            font_descriptors: &NSArray<NSFontDescriptor>,
            options: NSFontAssetRequestOptions,
        ) -> Retained<Self>;

        #[cfg(feature = "NSFontDescriptor")]
        #[method_id(@__retain_semantics Other downloadedFontDescriptors)]
        pub unsafe fn downloadedFontDescriptors(&self) -> Retained<NSArray<NSFontDescriptor>>;

        #[method_id(@__retain_semantics Other progress)]
        pub unsafe fn progress(&self) -> Retained<NSProgress>;

        #[cfg(feature = "block2")]
        #[method(downloadFontAssetsWithCompletionHandler:)]
        pub unsafe fn downloadFontAssetsWithCompletionHandler(
            &self,
            completion_handler: &block2::Block<dyn Fn(*mut NSError) -> Bool>,
        );
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl NSFontAssetRequest {
        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new() -> Retained<Self>;
    }
);
