//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ffi::*;
use objc2::__framework_prelude::*;

use crate::*;

mod private_NSObjectNSNibAwaking {
    pub trait Sealed {}
}

/// Category "NSNibAwaking" on [`NSObject`].
#[doc(alias = "NSNibAwaking")]
pub unsafe trait NSObjectNSNibAwaking:
    ClassType + Sized + private_NSObjectNSNibAwaking::Sealed
{
    extern_methods!(
        #[unsafe(method(awakeFromNib))]
        #[unsafe(method_family = none)]
        fn awakeFromNib(&self);

        #[unsafe(method(prepareForInterfaceBuilder))]
        #[unsafe(method_family = none)]
        fn prepareForInterfaceBuilder(&self);
    );
}

impl private_NSObjectNSNibAwaking::Sealed for NSObject {}
unsafe impl NSObjectNSNibAwaking for NSObject {}
